#!/bin/sh 

#########################################################################################
# Local variables defined
#########################################################################################

source /etc/gpio/GPIO_common

COM_SEL_SCRIPT_VERSION="1.2.1"
GPIO_COMSEL_SCRIPT_INITIALS="[GCS]"

#########################################################################################
# Fonctions
#########################################################################################

###########################################################################################
# Function that add the initials of the script to each message that will be logged.
# That makes the reading of the logs unambiguous

WriteLog() {
	LogUpdate "$GPIO_COMSEL_SCRIPT_INITIALS : $1" $2 $3
}

#########################################################################################
#########################################################################################
# Script beginning
#########################################################################################

#########################################################################################
# This script is called by a pollgpio function with the value of the gpio in entry
#
# Order of arguments
# $1 gpio value
#
# This script must update the content of a real file with the value given in entry
# The updating process must produce a MODIFY event which will be catched by the 
# application software using a watchservice. Therefore, the file is deleted before being 
# created again otherwise each overwritte of the file will produce 2 MODIFY events
# instead of the only one expected.

LogUpdate "----------------------------- $(basename "$0") v$COM_SEL_SCRIPT_VERSION started"

if [ -r "/sys/class/gpio/gpio234/value" ] && [ -r "/sys/class/gpio/gpio235/value" ]; then

	##### Get values from real and virtual for COM gpio
	MakesGpioRealPathAvailable $COM_DIR

	COM_REAL=0																													# Set a default value
	SEL_REAL=0																													# Set a default value

	if GetRealGpioValue $COM_DIR; then
		COM_REAL=$GPIO_VALUE
	fi

	if GetVirtualGpioValue $COM_DIR; then
		COM_VIRT=$GPIO_VALUE
	fi
	
	##### Get values from real and virtual for SEL gpio
	MakesGpioRealPathAvailable $SEL_DIR

	if GetRealGpioValue $SEL_DIR; then
		SEL_REAL=$GPIO_VALUE
	fi

	if GetVirtualGpioValue $SEL_DIR; then
		SEL_VIRT=$GPIO_VALUE
	fi

	##### Adjust content of real file with gpio value if necessary

	if [ $COM_VIRT != $COM_REAL ]; then
		LogUpdate "Adjusting content of gpio monitored file $COM_DIR with value $COM_VIRT"
		SetRealGpioValue $COM_DIR $COM_VIRT 
	fi

	if [ $SEL_VIRT != $SEL_REAL ]; then
		LogUpdate "Adjusting content of gpio monitored file $SEL_DIR with value $SEL_VIRT"
		SetRealGpioValue $SEL_DIR $SEL_VIRT 
	fi

	##### And do network stuff
	
	if [ ! -r "/etc/network/mode-router" ]; then
		LogUpdate "Stopping /etc/network/mode-router"
		echo "stop" > /etc/network/mode-router
	fi

	MODE=`cat /etc/network/mode-router`

	LogUpdate "COM+SEL : $COM_VIRT+$SEL_VIRT"

	if [ $COM_VIRT == "1" ]; then

		if [ $SEL_VIRT == "0" ]; then
			LogUpdate "Launching /etc/network/netconfig $MODE"
			/bin/sh /etc/network/netconfig.sh $MODE
		else
			LogUpdate "Launching /etc/network/netconfig normal"
			/bin/sh /etc/network/netconfig.sh normal
		fi

	else
		LogUpdate "Launching /etc/network/netconfig recovery"
		/bin/sh /etc/network/netconfig.sh recovery
	fi

else
	LogUpdate "ERROR ----- COM and SEL gpio files not found!"
fi

LogUpdate "----------------------------- $(basename "$0") v$COM_SEL_SCRIPT_VERSION ended"
