#! /bin/sh
#########################################################################################
# Versions
#
# 1.0.9
# Search and suppress and old dns-nameservers directive from network configuration files 
# in /etcv/network directory. This directive was a reminder of a sample file and added a
# false alternate dns-nameserver address
#
# 1.1.0
# Generic version of update and restore processes which use 2 generic functions :
#	- UpdateScript
#	- RestoreScript
#
# 1.1.1
# Adapt the script to a new a new file organization which reflect the target file 
# organization. Improve update function to be able to update without backup
# Add a Rsyslog modification
# Add an ifplugd modification
# Implement ReplaceUpdateDaemon function which was before in update.sh script.
#
# 1.1.2
# Add management of linkstate script which provides the state of all ethernet interfaces
# to the application through a monitored file
#
# 1.1.3
# Remove management of linkstate script as this method doesn't cover all cases and has 
# been replaced with a Network monitoring by the Prosyst framework
#
# 1.1.4
# Correct a problem on RestoreEnvironment function where the Restore_GPIOsFiles function
# was misspelled
#
# 1.1.5
# Add some router files to be changed in order to correct the DHCP server problem
#
# 1.2.0
# Add a specific part of logged msg so that a reader can clearly identify the emitter
# Modify some messages to make the whole process more understandable
# Make the common library mandatory to complete the whole process. Therefore source it
# from the update file at the process beginning
#
# 1.2.1
# Integrate EMSS specificities
#
# 1.2.2
# Totally change the way to kill all possibly running update_survey and inotifywait 
# processes before updating them
# 
# 1.2.3
# Add the copy of the interfaces-emss file to overwritte existing one which can contain
# a bad configuration
#
# 1.2.4
# Suppress command to manage INTERFACES_EMSS_FILE in Suppress_Bad_DNS as it generates an 
# useless error in logfile. This file is erased by the /etc/init.d/S11initspecific script
# and therefore don't have to ba managed anymore.
#
# 1.2.5
# Change some defines for the names to be more consistent
#
# 1.2.6
# Install Ngrok to provide distant access
# 
# 1.2.7
# Add chmod to ngrok to make it executable
#
# 1.2.8
# Install wancheck service
#
# 1.2.9
# Change behaviour of restore function as there is no real reason to downgrade the system. 
# Therefore nothing is done when function is called
#
# 1.2.10
# Install output_filtered_log script for better performance in logs
#
# 1.2.11
# Install S00pmic script
#########################################################################################

MODIFY_ENVIRONMENT_SCRIPT_VERSION="1.2.10"
MODIFY_ENVIRONMENT_SCRIPT_INITIALS="[ME-]"

source /usr/local/bin/update_common

#########################################################################################
# Global variables 
#########################################################################################

#------------------ Directories management

CURRENT_DIR="."

DIR_DATA="/data"
DIR_ROOT="/root"

DIR_BACKUP_ENV="$DIR_ROOT/backenv"

DIR_USR="usr"
DIR_USR_LOCAL="$DIR_USR/local"
DIR_USR_LOCAL_BIN="$DIR_USR_LOCAL/bin"

DIR_ETC="etc"
DIR_ETC_GPIO="$DIR_ETC/gpio"
DIR_ETC_INIT_D="$DIR_ETC/init.d"
DIR_ETC_NETWORK="$DIR_ETC/network"
DIR_ETC_IFPLUGD="$DIR_ETC/ifplugd"

#------------------ Files management

INTERFACES_FILE="interfaces"
INTERFACES_EMSS_FILE="interfaces-emss"
INTERFACES_INSTALLER_FILE="interfaces-installer"
INTERFACES_INTERCOM_FILE="interfaces-intercom"
INTERFACES_NORMAL_FILE="interfaces-normal"

UPDATE_COMMON_SCRIPT="update_common"
UPDATE_FROM_DIR="update_from_directory.sh"

SERIAL_USB_SCRIPT="serial_usb.sh"
INIT_BENCH_SCRIPT="S16BenchTest"
INIT_SPECIFIC_SCRIPT="S11initspecific"

INIT_S40NETWORK_SCRIPT="S40network"
NETCONFIG_SCRIPT="netconfig.sh"

INIT_S50SSHD_SCRIPT="S50sshd"
SSHD_CONFIG_FILE="sshd_config"

INIT_S85GPIO_SCRIPT="S85pollgpio"

ROUTER_INSTALLER_CFG="udhcpd-installer.conf"
ROUTER_INTERCOM_CFG="udhcpd-intercom.conf"

UPDATE_DAEMON_FILE="update_survey"
COMMON_LIBRARY="update_common"

INOTIFY_TSK="inotifywait"

NGROK="ngrok"

WANCHECK="wancheck"
INIT_S50WANCHECK_SCRIPT="S50wancheck"
WANCHECK_CFG="wancheck.conf"

OUTPUT_FILTERED_LOG_SCRIPT="output_filtered_log"
#------------------ Various

FUNC_RESTART="restart"
FUNC_INSTALL="Install"
FUNC_RESTORE="Restore"
FUNC_CLEAN="Clean"

BAD_ALTERNATE_DNS="dns-nameservers 192.168.0.250"

MAX_LOOP_KILL_DAEMON=2
MAX_TRIES=3
R_X_PERMISSIONS=555
PROTECT_FILE="PROTECTFILE"
UNPROTECT_FILE="UNPROTECTFILE"

INIT_PID=1
DBG_OUTPUT="/dev/console"

#------------- Libraries

UPDATE_COMMON_FILE="$DIR_USR_LOCAL_BIN/update_common"
UPDATE_TOOLS="$DIR_USR_LOCAL_BIN/BSPUpdateTools"

#------------------ LogFiles defines

LOG_DEBUG=1
LOG_WARNING=2
LOG_NORMAL=3

DATA_VAR_DIR=/data/var
DATA_VAR_LOG_DIR=$DATA_VAR_DIR/log
APP_LOGFILE=$DATA_VAR_LOG_DIR/update

#########################################################################################
# Functions
#########################################################################################

#########################################################################################
# Function that create a directory with a given name. Verify first that a file with the 
# same name doesn't exist at the designed location because it will prevent directory 
# creation. 
#
# Input - $1 Directory name
#
# IMPORTANT
# The "CreateDirectory" function also exists in "update_common" file but not since the 
# first version. So in case "update_common" can't be sourced (which is done at the very 
# beginning of the current script), this function will be used to log the error.

ModEnv_CreateDirectory() {
	
	if [ ! -z $1 ]; then 																											# The directory name must be given at the least 

		if [ ! -e "$1" ]; then																										# If directory already exists, just do nothing

			if [ -f "$1" ]; then 																									# Verify if a file with the same name already exists at current location
				rm "$1"																												# If so, it must be deleted, otherwise it will prevent the directory to be created
			fi

			mkdir "$1"
		fi
	fi
}

#########################################################################################
# Function that log a message in logfile
#
# Input - $1 Message
# 		  $2 Severity level of the message : NORMAL - WARNING
# 		  $3 Destination : LOGFILE - LOGFILE + CONSOLE
#
# IMPORTANT
# The "WriteLog" function also exists in "update_common" file but not since the 
# first version. So in case "update_common" can't be sourced (which is done at the very 
# beginning of the current script), this function will be used to log the error.

ModEnv_WriteLog() {

	# Create the log file and directories if needed

	if [ ! -e "$DATA_VAR_DIR" ]; then
		ModEnv_CreateDirectory $DATA_VAR_DIR
	fi

	if [ ! -e "$DATA_VAR_LOG_DIR" ]; then
		ModEnv_CreateDirectory $DATA_VAR_LOG_DIR
	fi

	LogDate=`date +%Y/%m/%d-%H:%M:%S`

	# The second parameter (optional) manage the message severity level
	# If not defined, the message is always written

	echo "$LogDate $MODIFY_ENVIRONMENT_SCRIPT_INITIALS : $1" >> $APP_LOGFILE
}

###########################################################################################
# Function that add the initials of the script to each message that will be logged.
# That makes the reading of the logs unambiguous

WriteLog() {
	LogUpdate "$MODIFY_ENVIRONMENT_SCRIPT_INITIALS : $1" $2 $3
}

###########################################################################################
# Function that set correct defines according to the current board type

ModEnv_InitSpecific() {
	boardtype=$(Get_BoardType)
 	LogUpdate "$boardtype board detected"
}

###########################################################################################
# Function that makes a copy from a file and verify its validity after the copy.
# Basically, it verify that the source fullname exists otherwise return an error, then it 
# copies it  using the destination fullname (without verifying that it already exists) and
# finally make a comparison between the source and the destination. If files differs, the
# process is restarted up to MAX_TRIES times before ending with an error.
#
# 	$1 : Source file fullname (dirname/filename)
# 	$2 : Destination file fullname (dirname/filename)
#
CopyAndVerify() {

	##### Make some useful verification before starting process

	if [ $# -ne 2 ]; then
		LogUpdate "ERROR ----- [CopyAndVerify] - $# arguments provided. Only source and destination fullnames expected. Abort"
		return 1
	fi

	if [ ! -f "$1" ]; then
		LogUpdate "ERROR ----- [CopyAndVerify] - Source fullname : $1 doesn't exists. Abort"
		return 1
	fi

	if [ -f "$2" ]; then
		LogUpdate "$2 already exists. File will be overwritted" $LOG_WARNING
	fi

	##### Then start a process with a limited tries

	verified=false
	difference=""
	boucle=0
	
	while [ $verified == false ] && [ $boucle -le $MAX_TRIES ]; do

		boucle=$((boucle+1))

		LogUpdate "Copy $1 to $2" $LOG_DEBUG
		cp "$1" "$2" 2>&1 >> $APP_LOGFILE

		##### Verify copy conformity

		difference=$(cmp "$1" "$2")

		if [ -z $difference ]; then
			verified=true
		else
			LogUpdate "Copies not identicals - difference = $difference" $LOG_WARNING
		fi

	done

	##### If MAX_TRIES reached without success, log it and return an error

	if [ $boucle -gt $MAX_TRIES ]; then
		LogUpdate "ERROR ----- Copy $1 on $2 can't be done for any reason. Max ($MAX_TRIES) tries reached. Abort copy"
		return 1
	fi

	return 0
}

###########################################################################################
ProtectionScript() {
	
	##### Makes some useful verifications before starting process

	if [ $# -ne 3 ]; then
		LogUpdate "ERROR ----- [ProtectionScript] - $# arguments provided. Arguments expected are : scriptname - dirname - action ($PROTECT_FILE / $UNPROTECT_FILE). Abort"
		return 1
	fi

	if [ ! -d "$2" ]; then
		LogUpdate "ERROR ----- [ProtectionScript] - dirname : $2 doesn't exists. Abort"
		return 1
	fi

	if [ ! -f "$2/$1" ]; then
		LogUpdate "ERROR ----- [ProtectionScript] - scriptname : $2/$1 doesn't exists. Abort"
		return 1
	fi

	scriptname=$1
	dirname=$2

	case "$3" in

		$PROTECT_FILE)
			LogUpdate "Protect $scriptname script" $LOG_DEBUG
			fileAttributes="+i"
			;;

		$UNPROTECT_FILE)
			LogUpdate "Unprotect $scriptname script" $LOG_DEBUG
			fileAttributes="-i"
			;;

		*)
			LogUpdate "ERROR ----- [ProtectionScript] - Action $3 not recognized. Expected $PROTECT_FILE / $UNPROTECT_FILE only. Abort"
			return 1
			;;

	esac

	##### As files that can be modified are accessible using an overlay, 
	##### it's necessary to switch on real directory to be able to do the 'chattr' function

	realFilePath=$DIR_DATA$dirname/$scriptname

	LogUpdate "chattr $fileAttributes $realFilePath" $LOG_DEBUG
	chattr "$fileAttributes" "$realFilePath" 2>&1 >> $APP_LOGFILE
 
	return 0
}

###########################################################################################
# Function that install a script
#
# Before installation, a potential previous version is first backuped (this backup is made 
# by a simple copy which doesn't depend on immutable attribute but neither maintain it). 
# Then the new script is copied from the source directory to its final location and its 
# permissions are setted to executable
#
#	$1 : scriptname
#	$2 : sourcedir
#	$3 : destinationdir
# 	$4 : backup dirname

InstallScript() {

	scriptname=$1
	sourcedir=$2
	destinationdir=$3
	backupdir=$4

	##### If an actual version of the script exist, make an optionnal backup and unprotect it
	if [ -f "$destinationdir/$scriptname" ]; then

		# Just log actual script version
		scriptVersion=$(CollectScriptVersion $destinationdir/$scriptname)
		LogUpdate "Current $scriptname script version : $scriptVersion"

		# An empty backup direcory just means that a backup is not wanted
		if [ ! -z "$backupdir" ]; then

			# Backup current script (Copy will not retain attributes)
			LogUpdate "Backuping .."
			if ! CopyAndVerify "$destinationdir/$scriptname" "$backupdir/$scriptname"; then
				return 1
			fi

		fi
	fi

	##### Then copy the script to its destination directory
	LogUpdate "Install $scriptname and verify its validity" $LOG_DEBUG
	if ! CopyAndVerify "$sourcedir/$scriptname" "$destinationdir/$scriptname"; then
		return 1
	fi 

	##### Now log the freshly installed script version
	scriptVersion=$(CollectScriptVersion $destinationdir/$scriptname)
	LogUpdate "New $scriptname script version : $scriptVersion"

	##### And finally set executable permissions to the script
	LogUpdate "Set permissions : $R_X_PERMISSIONS to $destinationdir/$scriptname script" $LOG_DEBUG
	chmod "$R_X_PERMISSIONS" "$destinationdir/$scriptname" 2>&1 >> $APP_LOGFILE

	return 0
}

###########################################################################################
# Function that update a script.
#
# Script means executable file. 
# Scripts are also sensible files that must be protected from accidental writes.
#
# Therefore when a script is updated, a potential previous version is first unprotected
# Then the new script is installed and protected using the immutable attribute.
#
#	$1 : scriptname
#	$2 : source dirname
# 	$3 : destination dirname
# 	$4 : backup dirname

InstallProtectedScript() {

	scriptname=$1
	sourcedir=$2
	destinationdir=$3
	backupdir=$4

	##### Considering that the update functionality is not dedicated for a particular version 
	##### and that ALL scripts must NOT be updated every time, it may happend that the function 
	##### could be called for a script that has NOT to be updated and therefore is simply not 
	##### present in the archive. This is NOT an error and must not be considered as such.

	if [ ! -f "$sourcedir/$scriptname" ]; then
		return 0
	fi

	##### OK the file exists, so start the process
	LogUpdate "-- Install and protect $destinationdir/$scriptname script"

	##### If an actual version of the script exist, make an optionnal backup and unprotect it
	if [ -f "$destinationdir/$scriptname" ]; then

		##### Unprotect the script as it must be overwritted
		if ! ProtectionScript "$scriptname" "$destinationdir" "$UNPROTECT_FILE"; then
			return 1
		fi

	fi

	##### Then install the new script
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		return 1
	fi

	###### And protect it
	if ! ProtectionScript "$scriptname" "$destinationdir" "$PROTECT_FILE"; then
		return 1
	fi

	LogUpdate "$scriptname installed and protected"
	return 0
}

###########################################################################################
# Function that restore a script.
#
# Script means executable file.
# Scripts are also sensible files that must be protected from accidental writes.
# Backup script are normal files without any attribute
#
# When a script is backuped, a potential existing script may be overwritted. Therefore we
# must first remove its immutable attribute otherwise neither overwritting operation nor
# delete action can be done. Then the backup file is copied from the backup directory to 
# its final location, its permissions are setted to executable and finally it's protected 
# using the immutable attribute
#
# 	$1 : scriptname
#	$2 : backup dirname
#	$3 : destination dirname

RestoreScript() {

	scriptname=$1
	backupdir=$2
	destinationdir=$3

	##### As the restore functionality is called automatically when an error occured during the
	##### update process, it may happen that we try to restore a file that has not been backuped
	##### This is not an error and must not be considered as such

	if [ ! -f "$backupdir/$scriptname" ]; then
		return 0
	fi

	##### Ok the backuped script exists, so start the process
	LogUpdate "-- Restoring $scriptname script"

	##### If an actual version of the script exists then unprotect it before restoring the backup
	if [ -f "$destinationdir/$scriptname" ]; then

		if ! ProtectionScript "$scriptname" "$destinationdir" "$UNPROTECT_FILE"; then
			return 1
		fi

	fi

	##### Then restore the backuped script
	if ! InstallScript "$scriptname" "$backupdir" "$destinationdir"; then
		return 1
	fi

	##### And protect it
	if ! ProtectionScript "$scriptname" "$destinationdir" "$PROTECT_FILE"; then
		return 1
	fi

	LogUpdate "$scriptname restored"
	return 0
}

###########################################################################################
# Create the log file and associated directories if needed

MakeBackupDirectories() {

	backupdir="$DIR_BACKUP_ENV"

	if [ ! -e "$backupdir" ]; then
		CreateDirectory $backupdir 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_ETC" ]; then
		CreateDirectory "$backupdir/$DIR_ETC" 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_ETC_GPIO" ]; then
		CreateDirectory "$backupdir/$DIR_ETC_GPIO" 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_ETC_INIT_D" ]; then
		CreateDirectory "$backupdir/$DIR_ETC_INIT_D" 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_USR" ]; then
		CreateDirectory "$backupdir/$DIR_USR" 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_USR_LOCAL" ]; then
		CreateDirectory "$backupdir/$DIR_USR_LOCAL" 2>&1 >> $APP_LOGFILE
	fi

	if [ ! -e "$backupdir/$DIR_USR_LOCAL_BIN" ]; then
		CreateDirectory "$backupdir/$DIR_USR_LOCAL_BIN" 2>&1 >> $APP_LOGFILE
	fi
}

###########################################################################################
Suppress_Bad_DNS() {

	if [ -z $1 ] || [ -z $2 ]; then
		LogUpdate "ERROR managing DNS. Missing arguments"
		return 0
	fi

	dirname=$1
	filename=$2

	if [ ! -e $dirname/$filename ]; then
		LogUpdate "ERROR managing DNS. File $dirname/$filename doesn't exists"
		return 0
	fi

	##### Suppress a line which was a reminder of a sample from all network configuration files
	if grep -q "dns-nameservers 192.168.0.250" $dirname/$filename; then
		LogUpdate "-- Suppress obsolete dns-nameservers directive in $dirname/$filename"
		sed -i '/dns-nameservers 192.168.0.250/d' $dirname/$filename
	fi

	return 0
}

###########################################################################################
# Physically update the common library file. 
# Making a backup is a good idea in case of environment restoration

Update_CommonLib() {

	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	backupdir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	scriptname="$UPDATE_COMMON_SCRIPT"

	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Restore_CommonLib() {

	sourcedir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	scriptname="$UPDATE_COMMON_SCRIPT"
	
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	##### Reload it 
	LogUpdate "Reload $destinationdir/$scriptname"
	source "$destinationdir/$scriptname"
}

###########################################################################################
Update_AnalyzeUSB() {

	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	backupdir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	scriptname="$UPDATE_FROM_DIR"

	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Restore_AnalyzeUSB() {

	sourcedir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	scriptname="$UPDATE_FROM_DIR"

	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Replace_Pollgpio_InitScript() {

	# Replace pollgpio init script
	sourcedir="./$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"
	scriptname="$INIT_S85GPIO_SCRIPT"

	LogUpdate "-- Install $destinationdir/$scriptname file"

	##### Update the script without making a backup as it must be changed anyway
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Replacement of $destinationdir/$scriptname failed"
		return 1
	fi

	LogUpdate "$scriptname installed"
	
	return 0
}

###########################################################################################
Update_GPIOsFiles() {

	##### First of all, install the correct gpios files

	workingdir="$DIR_ETC_GPIO"

	sourcedir="./$workingdir"
	destinationdir="/$workingdir"
	backupdir="$DIR_BACKUP_ENV/$workingdir"
	
	scriptname="CAPGOOD.sh"
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	scriptname="COM_SEL.sh"
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	scriptname="PFAIL.sh"
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	scriptname="POE_OK.sh"
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	scriptname="GPIO_common"
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	##### Then the pollgpio init script
	if ! Replace_Pollgpio_InitScript; then
		return 1
	fi

	return 0
}

###########################################################################################
Restore_GPIOsFiles() {

	sourcedir="$DIR_BACKUP_ENV/$DIR_ETC_GPIO"
	destinationdir="/$DIR_ETC_GPIO"

	scriptname="CAPGOOD.sh"
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	scriptname="COM_SEL.sh"
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	scriptname="PFAIL.sh"
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	scriptname="POE_OK.sh"
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	scriptname="GPIO_common"
	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Update_ApplicationScript() {

	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	backupdir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	scriptname="$HAGER_APP_STARTING_SCRIPT"

	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Restore_ApplicationScript() {

	sourcedir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	scriptname="$HAGER_APP_STARTING_SCRIPT"

	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	return 0
}

###########################################################################################
Modify_Router_Mode() {

	# Modify network init script
	sourcedir="./$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"
	scriptname="$INIT_S40NETWORK_SCRIPT"

	LogUpdate "-- Install $destinationdir/$scriptname file"

	# Replace the script without making a backup as it must be changed anyway
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi

	LogUpdate "$scriptname installed"
	
	# Modify netconfig script
	sourcedir="./$DIR_ETC_NETWORK"
	destinationdir="/$DIR_ETC_NETWORK"
	scriptname="$NETCONFIG_SCRIPT"

	LogUpdate "-- Install $destinationdir/$scriptname file"
	
	# Replace the script without making a backup as it must be changed anyway
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi

	LogUpdate "$scriptname installed"
	
	return 0
}

###########################################################################################
Modify_Router_Config() {

	# Modify router installer configuration file
	sourcedir="./$DIR_ETC"
	destinationdir="/$DIR_ETC"
	conf_filename="$ROUTER_INSTALLER_CFG"

	##### Then just copy (overwrite) the new router-installer configuration file
	LogUpdate "-- Replace $conf_filename configuration file"

	if ! CopyAndVerify "$sourcedir/$conf_filename" "$destinationdir/$conf_filename"; then
		return 1
	fi 

	LogUpdate "$conf_filename replaced"

	# Modify router intercom configuration file
	sourcedir="./$DIR_ETC"
	destinationdir="/$DIR_ETC"
	conf_filename="$ROUTER_INTERCOM_CFG"

	##### Then just copy (overwrite) the new router-intercom configuration file
	LogUpdate "-- Replace $conf_filename configuration file"

	if ! CopyAndVerify "$sourcedir/$conf_filename" "$destinationdir/$conf_filename"; then
		return 1
	fi 

	LogUpdate "$conf_filename replaced"

	return 0
}

###########################################################################################
Modify_Router_Config_And_Mode() {

	if ! Modify_Router_Mode; then
		LogUpdate "Can't modify network router scripts. Abort"
		return 1
	fi

	if ! Modify_Router_Config; then
		LogUpdate "Can't modify router configuration files. Abort"
		return 1
	fi

	return 0
}

###########################################################################################
Update_BenchTest() {

	# Update benchtest itself
	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
	backupdir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	scriptname="$SERIAL_USB_SCRIPT"

	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	# Update benchtest launcher
	sourcedir="./$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"
	backupdir="$DIR_BACKUP_ENV/$DIR_ETC_INIT_D"
	scriptname="$INIT_BENCH_SCRIPT"

	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir" "$backupdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	return 0
}

Install_i2setTools(){

	mount -o remount,rw /dev/root / && tar xvf ./i2cset.tar -C / 
	if [ $? == 0 ]; then
		echo "PMIC patch and i2c-tools has been deployed"  2>&1 >> $APP_LOGFILE
	else 
		echo "PMIC patch and i2c_tools has not been deployed" 2>&1 >> $APP_LOGFILE
	fi
	mount -o remount,ro /dev/root /

}


#########################################################################################
Restore_BenchTest() {

	# Restore benchtest itself
	scriptname="$SERIAL_USB_SCRIPT"
	sourcedir="$DIR_BACKUP_ENV/$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"

	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	# Restore benchtest launcher
	scriptname="$INIT_BENCH_SCRIPT"
	sourcedir="$DIR_BACKUP_ENV/$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"

	if ! RestoreScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Restoration of $scriptname failed"
		return 1
	fi

	return 0
}

#########################################################################################
Install_InitSpecific() {

	# Update script which manage platform specificities
	sourcedir="./$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"
	backupdir="$DIR_BACKUP_ENV/$DIR_ETC_INIT_D"
	scriptname="$INIT_SPECIFIC_SCRIPT"

	# No backup made
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		LogUpdate "ERROR ----- Update of $scriptname failed"
		return 1
	fi

	return 0
}

#########################################################################################
Update_SSHConfig() {

	if [ $boardtype == $EMSS_BOARD ]; then

		sourcedir="./$DIR_ETC"
		destinationdir="/$DIR_ETC"
		sourcefile="sshd_config-emss"

		LogUpdate "-- Replace $SSHD_CONFIG_FILE file"

		##### Then just copy (overwrite) the new rsyslog configuration file
		LogUpdate "Copy $sourcedir/$sourcefile over $destinationdir/$SSHD_CONFIG_FILE"

		if ! CopyAndVerify "$sourcedir/$sourcefile" "$destinationdir/$SSHD_CONFIG_FILE"; then
			return 1
		fi 

		LogUpdate "$SSHD_CONFIG_FILE replaced"

		##### And finally restart the rsyslog daemon
		LogUpdate "Restart SSH ssh service - /$DIR_ETC_INIT_D/$INIT_S50SSHD_SCRIPT $FUNC_RESTART"
		"/$DIR_ETC_INIT_D/$INIT_S50SSHD_SCRIPT" $FUNC_RESTART
	
	fi

	return 0
}

###########################################################################################
Modify_Interfaces_Files() {

	destinationdir="/$DIR_ETC_NETWORK"

	if [ $boardtype == $KNX_BOARD ]; then

		Suppress_Bad_DNS $destinationdir $INTERFACES_FILE
		Suppress_Bad_DNS $destinationdir $INTERFACES_INSTALLER_FILE
		Suppress_Bad_DNS $destinationdir $INTERFACES_INTERCOM_FILE
		Suppress_Bad_DNS $destinationdir $INTERFACES_NORMAL_FILE

	fi

	if [ $boardtype == $EMSS_BOARD ]; then

		sourcedir="./$DIR_ETC_NETWORK"
		sourcefile="interfaces-emss"

		# Do something only if a specific file is present
		if [ -e $sourcedir/$sourcefile ]; then

			LogUpdate "-- Replace $destinationdir/$INTERFACES_FILE file"

			# Then just copy (overwrite) the new interfaces file
			LogUpdate "Copy $sourcedir/$sourcefile over $destinationdir/$INTERFACES_FILE"

			cc=$(grep -in eth1 "/$destinationdir/$INTERFACES_FILE")
			LogUpdate "Before writing $destinationdir/$INTERFACES_FILE : $cc" $LOG_DEBUG

			if ! CopyAndVerify "$sourcedir/$sourcefile" "$destinationdir/$INTERFACES_FILE"; then
				return 1
			fi 

			LogUpdate "$INTERFACES_FILE replaced"

			cc=$(grep -in eth1 "/$destinationdir/$INTERFACES_FILE")
			LogUpdate "After writing $destinationdir/$INTERFACES_FILE : $cc" $LOG_DEBUG

			# Copy also specific EMSS interfaces file as it, as in version older than 2.0.0
			# an interfaces-emss file was already there but with a bad configuration.
			# This file will be used in the /etc/initspecific script to overwrite system resource

			LogUpdate "Copy $sourcedir/$sourcefile over $destinationdir/$sourcefile"

			if ! CopyAndVerify "$sourcedir/$sourcefile" "$destinationdir/$sourcefile"; then
				return 1
			fi 

			# And finally restart the network
			LogUpdate "Restart network service - /$DIR_ETC_INIT_D/$INIT_S40NETWORK_SCRIPT $FUNC_RESTART"
			
			"/$DIR_ETC_INIT_D/$INIT_S40NETWORK_SCRIPT" $FUNC_RESTART

		fi
	fi

	return 0
}

#########################################################################################
Modify_Rsyslog() {

	launcher_sourcedir="./$DIR_ETC_INIT_D"
	launcher_destinationdir="/$DIR_ETC_INIT_D"

	conf_sourcedir="./$DIR_ETC"
	conf_destinationdir="/$DIR_ETC"

	launcher_filename="S01rsyslog"
	conf_filename="rsyslog.conf"

	##### Update the script without making a backup as the launcher must be updated anyway
	if ! InstallProtectedScript "$launcher_filename" "$launcher_sourcedir" "$launcher_destinationdir"; then
		LogUpdate "ERROR ----- Update of $launcher_filename failed"
		return 1
	fi

	##### Then just copy (overwrite) the new rsyslog configuration file
	LogUpdate "-- Replace $conf_filename file"

	if ! CopyAndVerify "$conf_sourcedir/$conf_filename" "$conf_destinationdir/$conf_filename"; then
		return 1
	fi 

	##### And finally restart the rsyslog daemon
	LogUpdate "Restarting $launcher_destinationdir/$launcher_filename daemon"
	"$launcher_destinationdir/$launcher_filename" $FUNC_RESTART

	LogUpdate "$conf_filename replaced"
	return 0
}

#########################################################################################
Modify_Ifplugd() {

	scriptname="ifplugd.action"
	sourcedir="./$DIR_ETC_IFPLUGD"
	destinationdir="/$DIR_ETC_IFPLUGD"

	##### Just copy (overwrite) the new ifplugd.action file
	LogUpdate "-- Replace $destinationdir/$scriptname file"
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi

	LogUpdate "$scriptname replaced"
	return 0
}

#########################################################################################

Kill_All_InotifyWaitRunning() {

	LogUpdate "Look at $INOTIFY_TSK processes running"
		
	# Get a string with all inotifywait's pid currently running
	allPidInotify=$(GetRunningIds $INOTIFY_TSK)

	if [ -z $allPidInotify ]; then
		LogUpdate "No $INOTIFY_TSK process found"
		return 0
	fi

	for pidInotify in $allPidInotify; do
	
		# Set by default, will be changed later if necessary
		killpid=false

		# Verify if this process is a child of an unwanted update daemon duplication
		LogUpdate "$INOTIFY_TSK process $pidInotify found. Looking for it's parent id" > $LOG_DEBUG

		# Get parent pid of inotifywait process
		ppidInotify=$(GetProcessParentId $pidInotify)
		retCode=$?

		if [ $retCode -ne 0 ]; then
			return 1
		fi

		# Ok parent pid is retrieved, verify now that is not a lonely process
		if [ $ppidInotify -eq $INIT_PID ]; then

			LogUpdate "Parent pid is $INIT_PID. InotifyWait process $pidInotify is an orphan process that can be killed alone"

			GentlyKillProcess $pidInotify

			if ! DidProcessDied $pidInotify; then

				LogUpdate "$INOTIFY_TSK process $pidInotify can't be gently killed. Hard way now !" $LOG_WARNING
				HardlyKillprocess $pidInotify

				if ! DidProcessDied $pidInotify; then
					LogUpdate "ERROR ----- Not able to correctly kill the $INOTIFY_TSK (pid $pidInotify) process"
					return 1
				fi
			fi

			LogUpdate "$INOTIFY_TSK process $pidInotify has been killed"

		else

			# Inotify parent is NOT the init process. Let it alive
			var=$(cat /proc/$ppidInotify/cmdline)
			LogUpdate "Parent of $pidInotify pid is the $var process. It will NOT be killed"

		fi

 	done

}

#########################################################################################
# Function that is in charge of killing every update_survey processes currently running.
# As in previsous versions, an update_survey process can be the father of another 
# update_survey generated by the pipe used inside the main loop, the idea is to process
# in two loops.
# The first one will kill only the processes that are attached to the INIT process and 
# therefore can be considered as parent. Once killed, their children will become orphan 
# and will be in turn attached to the INIT process too. 
# A second loop will killed these orphans and will normally clean every remaining 
# update_process currently running.

Kill_All_SurveyRunning() {

	bCont=true
	difference=""
	boucle=1
	
	while [ $bCont == true ] && [ $boucle -le $MAX_LOOP_KILL_DAEMON ]; do

		LogUpdate "Look at $UPDATE_DAEMON_FILE processes running in pass $boucle"

		# Get a string with all update daemon's pid currently running
		allPidSurvey=$(GetRunningIds $UPDATE_DAEMON_FILE)

		if [ -z $allPidSurvey ]; then
			LogUpdate "No $UPDATE_DAEMON_FILE process found"
			bCont=false
			continue
		fi

		# look at every pids to manage only parent processes
		for pidSurvey in $allPidSurvey; do

			LogUpdate "$UPDATE_DAEMON_FILE pid:$pidSurvey detected"

			# Get parent pid of survey process
			ppidSurvey=$(GetProcessParentId $pidSurvey)
			retCode=$?

			if [ $retCode -ne 0 ]; then
				return 1
			fi

			# Ok parent pid is retrieved. Just kill those which have no parent (attached to INIT)
			if [ $ppidSurvey -eq $INIT_PID ]; then

				GentlyKillProcess $pidSurvey

				if ! DidProcessDied $pidSurvey; then

					LogUpdate "$UPDATE_DAEMON_FILE process $pidSurvey can't be gently killed. Hard way now !" $LOG_WARNING
					HardlyKillprocess $pidSurvey

					if ! DidProcessDied $pidSurvey; then
						LogUpdate "ERROR ----- Not able to correctly kill the $UPDATE_DAEMON_FILE (pid $pidSurvey) process"
						return 1
					fi
				fi

				LogUpdate "$UPDATE_DAEMON_FILE process $pidSurvey has been killed"

			else
				LogUpdate "$UPDATE_DAEMON_FILE process $pidSurvey don't have to be killed. It has process $ppidSurvey as parent"

			fi

		done

		# Do another loop to kill remaining processes
		boucle=$((boucle+1))

	done

	return 0
}

###########################################################################################
# Function that replace the update_daemon with a version which allows more time (120s) to
# the application to validate its start by writing the platformstate/started file.
# The version flashed with 1.3.2 firmware just allowed 60s for it and that is not enough 
# under certain network conditions (no DHCP server)

ReplaceUpdateDaemon() {

	scriptname="$UPDATE_DAEMON_FILE"
	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"

	# Verify first that the update daemon script exist at least
	if [ ! -f "$destinationdir/$scriptname" ]; then
		LogUpdate "No $destinationdir/$scriptname script found !"
		return 1
	fi

	# First, kill all update_survey processes currently running
	LogUpdate "-- Kill all $scriptname processes currently running"
	
	if ! Kill_All_SurveyRunning; then
		LogUpdate "Impossible to kill all $scriptname processes currently running. Abort"
	fi

	# Now kill the remaining inotifywait process if some
	LogUpdate "-- Kill all $INOTIFY_TSK processes currently running"

	if ! Kill_All_InotifyWaitRunning; then
		LogUpdate "Impossible to kill all $INOTIFY_TSK processes currently running. Abort"
	fi

	# Verify that there is not an update_survey running process anymore but should be not
	if IsProcessRunning $UPDATE_DAEMON_FILE; then

		LogUpdate "Unwanted running $UPDATE_DAEMON_FILE process. Try to kill it"
		if ! KillRunningProcess $UPDATE_DAEMON_FILE; then
			return 1
		fi

		LogUpdate "$UPDATE_DAEMON_FILE process killed"
	fi

	# Verify that there is not an inotifywait running process anymore but should be not
	if IsProcessRunning $INOTIFY_TSK; then

		LogUpdate "Unwanted running $INOTIFY_TSK process. Try to kill it"
		if ! KillRunningProcess $INOTIFY_TSK; then
			return 1
		fi

		LogUpdate "$INOTIFY_TSK process killed"
	fi

	# Then overwrite the daemon script by the new one provided without making a backup
	if ! InstallProtectedScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi

	# And finally restart it 
	currentdir=$(pwd)

	cd $destinationdir

	LogUpdate "Restart $scriptname script from $(pwd)"
	nohup sh ./$scriptname > $DBG_OUTPUT &

	cd $currentdir
	return 0
}

#########################################################################################
# Function that install missing scripts or overwrite existing ones that are not working 
# correctly. Action may rely on script version if found.

Correct_Bugs_From_Previous_Versions() {

	# Correct the network router mode bugs
	Modify_Router_Config_And_Mode

	# Modify interface files depending on board type
	if ! Modify_Interfaces_Files; then
		return 1
	fi

	# Modify Rsyslog launcher and configuration
	if ! Modify_Rsyslog; then
		return 1
	fi

	# Modify ifplugd action file
	if ! Modify_Ifplugd; then
		return 1
	fi

	return 0
}

#########################################################################################
Install_Ngrok() {

	scriptname=$NGROK
	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"


	# Install Ngrok file
	LogUpdate "Install $destinationdir/$scriptname from $sourcedir/$scriptname"
	if ! CopyAndVerify "$sourcedir/$scriptname" "$destinationdir/$scriptname"; then
		return 1
	fi

	##### And finally set executable permissions to the script
	LogUpdate "Set permissions : $R_X_PERMISSIONS to $destinationdir/$scriptname script" $LOG_DEBUG
	chmod "$R_X_PERMISSIONS" "$destinationdir/$scriptname" 2>&1 >> $APP_LOGFILE

	return 0
}

#########################################################################################
Install_Wancheck() {

	## Install executable script - InstallScript
    scriptname=$WANCHECK
	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"
    
    LogUpdate "Install $destinationdir/$scriptname from $sourcedir/$scriptname"
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi
    
	## Install configuration file - CopyAndVerify
    conf_filename=$WANCHECK_CFG
    sourcedir="./$DIR_ETC"
    destinationdir="/$DIR_ETC"
    
    LogUpdate "Install $destinationdir/$conf_filename from $sourcedir/$conf_filename"
    if ! CopyAndVerify "$sourcedir/$conf_filename" "$destinationdir/$conf_filename"; then
		return 1
	fi 
	
	## Install /etc/init.d script starter - InstallScript
    scriptname=$INIT_S50WANCHECK_SCRIPT
	sourcedir="./$DIR_ETC_INIT_D"
	destinationdir="/$DIR_ETC_INIT_D"
    
    LogUpdate "Install $destinationdir/$scriptname from $sourcedir/$scriptname"
	if ! InstallScript "$scriptname" "$sourcedir" "$destinationdir"; then
		return 1
	fi
    
    ## (Re)start Wancheck service
    LogUpdate "Restart SSH ssh service - /$DIR_ETC_INIT_D/$INIT_S50WANCHECK_SCRIPT $FUNC_RESTART"
	"/$DIR_ETC_INIT_D/$INIT_S50WANCHECK_SCRIPT" $FUNC_RESTART
    
    return 0
}

#########################################################################################
Install_Output_Filtered_Log() {

	scriptname=$OUTPUT_FILTERED_LOG_SCRIPT
	sourcedir="./$DIR_USR_LOCAL_BIN"
	destinationdir="/$DIR_USR_LOCAL_BIN"


	# Install file
	echo "Install_Output_Filtered_Log" >> $APP_LOGFILE
	LogUpdate "Install $destinationdir/$scriptname from $sourcedir/$scriptname"
	if ! CopyAndVerify "$sourcedir/$scriptname" "$destinationdir/$scriptname"; then
		return 1
	fi

	##### And finally set executable permissions to the script
	LogUpdate "Set permissions : $R_X_PERMISSIONS to $destinationdir/$scriptname script" $LOG_DEBUG
	chmod "$R_X_PERMISSIONS" "$destinationdir/$scriptname" 2>&1 >> $APP_LOGFILE

	return 0
}

#########################################################################################
# Function that source the new common lib which must be provided with the update zipfile

ModEnv_Source_New_Common_Lib() {

	ModEnv_WriteLog "Common library version actually running : $UPDATE_COMMON_SCRIPT_VERSION"

	# Look at new common library's presence

	if [ ! -f "$DIR_UPDATE_ENV/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY" ]; then
		ModEnv_WriteLog "ERROR ----- $DIR_UPDATE_ENV/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY can't be found !!"
		return 1
	fi

	# Install output_filtered_log
	Install_Output_Filtered_Log

	# Everything fine, just source the new common library so that we can use its functionalities

	ModEnv_WriteLog "Sourcing $DIR_UPDATE_ENV/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY"
	source $DIR_UPDATE_ENV/$DIR_USR_LOCAL_BIN/$COMMON_LIBRARY

	ModEnv_WriteLog "Now running with library version : $UPDATE_COMMON_SCRIPT_VERSION"

	return 0
}

#########################################################################################
InstallEnvironment() {

	##### First of all, source the common library that must be included in update zip file
	if ! ModEnv_Source_New_Common_Lib; then
		ModEnv_WriteLog "ERROR ----- Modify environment aborted !!"
		exit 
	fi

	##### Then set specific defines according to current hardware
	ModEnv_InitSpecific

	##### Then start the new environment installation
	LogUpdate "Install environment"

	# Create backup environment
	MakeBackupDirectories

	# Process the whole environment modification
	if ! Update_CommonLib; then
		return 1
	fi

	Update_AnalyzeUSB
	Update_GPIOsFiles
	Update_ApplicationScript

	Update_SSHConfig

	# Install the bench test scripts
	Update_BenchTest

	# Install the init platform specificities script
	Install_InitSpecific

	# Install ngrok to provide distant access
	Install_Ngrok

	# Install WAN check
	Install_Wancheck

	# Make some adjustements in relation with old versions
	Correct_Bugs_From_Previous_Versions
	
	# Install i2c-set tools
	Install_i2setTools

	# Replace and restart update_survey daemon
	if ! ReplaceUpdateDaemon; then
		return 1
	fi

	LogUpdate "Install environment ended"
}

#########################################################################################
RestoreEnvironment() {

	LogUpdate "Restore environment"

	LogUpdate "Nothing done"

	# Restore the whole environment
#	Restore_AnalyzeUSB
#	Restore_GPIOsFiles
#	Restore_ApplicationScript

	# Restore old bench test
#	Restore_BenchTest

	# Restore old capabilities
#	Restore_CommonLib

	LogUpdate "Restore environment ended"
}

#########################################################################################
CleanEnvironment() {

	# Nothing specific to do
	return 0
}

#########################################################################################
#########################################################################################
# Script entry
#########################################################################################
#########################################################################################

ModEnv_WriteLog "----------------------------- $(basename "$0") ($1) - v$MODIFY_ENVIRONMENT_SCRIPT_VERSION (using library $UPDATE_COMMON_SCRIPT_VERSION) - started"

##### Initialize defines that depends on the location of the script which is de facto the update "rootfs"

DIR_UPDATE_ENV=$(pwd)

##### Then, look at the job to do

if [ ! -z "$1" ]; then

	case "$1" in

		$FUNC_INSTALL)
			InstallEnvironment
			;;

		$FUNC_RESTORE)
			RestoreEnvironment
			;;

		$FUNC_CLEAN)
			CleanEnvironment
			;;

		*)
			LogUpdate "ERROR ----- Parameter not recognized : $1 !. Script aborted"
			;;

	esac

else
	LogUpdate "ERROR ----- Missing parameter to launch the script. Needs $FUNC_INSTALL or $FUNC_RESTORE !"
fi

LogUpdate "----------------------------- $(basename "$0") - v$MODIFY_ENVIRONMENT_SCRIPT_VERSION - ended"
