#! /bin/sh
#############################################################################
# install-firmware.sh
#
# Script to install a (verified) downloaded rootfs.ext2 on a free partition
#
# 2014 Logilin for FDE
# 20/10/2014 : Modified by FDE <a.courrier@fde.fr>
#############################################################################

HBOX_UPDATE_LOG_FILE=/data/var/log/update

EXEC_NAME=`basename $0`
MP_4_PARTFS="/tmp/pfs"
MP_4_ROOTFS="/tmp/rfs"


SMART_BOOT="/usr/local/bin/smart-boot.sh"
ROOTFS_EXT2_PATH="/tmp/rootfs.ext2"

PART_RESCUE_NAME="mmcblk0p3"
PART_SYS1_NAME="mmcblk0p5"
PART_SYS2_NAME="mmcblk0p6"


PART_SYS1_FLAG=`fw_printenv | grep "part_sys1_flag" | awk -F= '{print $2}'`
PART_SYS2_FLAG=`fw_printenv | grep "part_sys2_flag" | awk -F= '{print $2}'`

PART_SYS1_IS_VALID=`echo $PART_SYS1_FLAG | grep "OK" | wc -l`
PART_SYS2_IS_VALID=`echo $PART_SYS2_FLAG | grep "OK" | wc -l`

PART_SYS1_IS_BACKUP=`echo $PART_SYS1_FLAG | grep "BACKUP" | wc -l`
PART_SYS2_IS_BACKUP=`echo $PART_SYS2_FLAG | grep "BACKUP" | wc -l`

# Find partition where install new BSP
CurrentPart=`cat /proc/cmdline | awk -F"root=/dev/" '{print $2}' | awk -F' ' '{print $1}'`

source /usr/local/bin/BSPUpdateTools

# Check if rootfs.ext file is present
if [[ ! -f $ROOTFS_EXT2_PATH ]] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (file $ROOTFS_EXT2_PATH not found)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: File $ROOTFS_EXT2_PATH not found !!"
	update_error
	exit 1
fi

# Check which system partition is currently used and choose the other one 
# or choose the first system partition if we use the rescue part or anything else (ie. nfs)
if [ "$CurrentPart" = "$PART_SYS1_NAME" ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Install new firmware on \"system_2\" partition" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: \"system_1\" partition is currently used, then use \"system_2\" partition to install new BSP" > /dev/console
	NewBSPPart=$PART_SYS2_NAME
elif [ "$CurrentPart" = "$PART_SYS2_NAME" ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Install new firmware on \"system_1\" partition" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: \"system_2\" partition is currently used, then use \"system_1\" partition to install new BSP" > /dev/console
	NewBSPPart=$PART_SYS1_NAME
#elif [ "$CurrentPart" = "$PART_RESCUE_NAME" ] ; then
#	echo "$EXEC_NAME: \"rescue\" partition is currently used, then check which is the best partition between \"system_1\" and \"system_2\"" > /dev/console
	# Here we have to check which is the best partition to install new BSP
#	if [[ $PART_SYS1_IS_VALID -eq 1 && $PART_SYS2_IS_VALID -eq 1 ]] ; then
		# system_1 && system_2 are flagged as "OK" then use system_1
#		echo "$EXEC_NAME: \"system_1\" and \"system_2\" are flaged as \"OK\", then use \"system_2\" partition to install new BSP" > /dev/console
#       NewBSPPart=$PART_SYS2_NAME
#	elif [ $PART_SYS1_IS_VALID -eq 1 ] ; then
		# system_1 is valid then use system_2
#		echo "$EXEC_NAME: \"system_1\" is flaged as \"OK\", then use \"system_2\" partition to install new BSP" > /dev/console
#		NewBSPPart=$PART_SYS2_NAME
#	elif [ $PART_SYS2_IS_VALID -eq 1 ] ; then
		# system_2 is valid the use system_1
#		echo "$EXEC_NAME: \"system_2\" is flaged as \"OK\", then use \"system_1\" partition to install new BSP" > /dev/console
#        NewBSPPart=$PART_SYS1_NAME
#	elif [ $PART_SYS1_IS_BACKUP -eq 1 ] ; then 
		# system_1 is backup and system_2 is not OK then use system_2
#		echo "$EXEC_NAME: \"system_1\" partition is flaged as \"BACKUP\" and \"system_2\" is flaged as \"($PART_SYS2_FLAG)\", then use \"system_2\" partition to install new BSP" > /dev/console
#		NewBSPPart=$PART_SYS2_NAME
#	elif [ $PART_SYS2_IS_BACKUP -eq 1 ] ; then
		# system_2 is backup and system_1 is not OK then use system_1	
#		echo "$EXEC_NAME: \"system_2\" partition is flaged as \"BACKUP\" and \"system_1\" is flaged as \"($PART_SYS1_FLAG)\", then use \"system_1\" partition to install new BSP" > /dev/console
#		NewBSPPart=$PART_SYS1_NAME
#	else
		# system_1 and system_2 are not OK then use system_1
#		echo "$EXEC_NAME: \"system_1\" is flaged as \"($PART_SYS1_FLAG)\" and \"system_2\" is flaged as \"($PART_SYS2_FLAG)\", then use \"system_1\" partition to install new BSP" \
#		> /dev/console
#		NewBSPPart=$PART_SYS1_NAME
#	fi
else
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Install new firmware on \"system_1\" partition (unknown current partition)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: unknown which partition is currently used ($CurrentPart), then use \"system_1\" partition to install new BSP" > /dev/console
	NewBSPPart=$PART_SYS1_NAME
fi

# Set new partition name
if [ "$NewBSPPart" = "$PART_SYS1_NAME" ] ; then
	PartNum=1
elif [ "$NewBSPPart" = "$PART_SYS2_NAME" ] ; then
	PartNum=2
else
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot define partition number)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: ERROR !!! Cannot define partition name" > /dev/console
	update_error
	exit 1
fi

# Do some verification
# - is new partition a block device ?
if [[ ! -b /dev/$NewBSPPart ]] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (/dev/$NewBSPPart is not a block device)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: /dev/$NewBSPPart is not a block device" > /dev/console
	update_error
	exit 1
fi
# - is new partition the current partition
if [ "$NewBSPPart" = "$CurrentPart" ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot update partition which is currently used)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: Cannot update current partition (/dev/$NewBSPPart)" > /dev/console
	update_error
	exit 1
fi

# Here we know which partition we have to use to install new BSP
#echo "$EXEC_NAME: New BSP will be installed on \"system_$PartNum\" \"(/dev/$NewBSPPart\")" > /dev/console

# Set flag of new partition
set_partition_as_lock $PartNum
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot Lock Partition)" >> $HBOX_UPDATE_LOG_FILE
	update_error
	exit 1
fi

# umount new partition (just in case)
umount /dev/$NewBSPPart > /dev/null 2>&1 

# Format New Partition
mkfs.ext3 -L system_$PartNum /dev/$NewBSPPart > /dev/console
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot Format Partition)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: unable to format partition /dev/$NewBSPPart" > /dev/console
	update_error
	exit 1
fi

# Create directories where partition and HBoxFirmware file will be mounted
mkdir -p $MP_4_PARTFS > /dev/console
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot create $MP_4_PARTFS directory)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: unable to create $MP_4_PARTFS directory" > /dev/console
	update_error
	exit 1
fi

mkdir -p $MP_4_ROOTFS > /dev/console
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot create $MP_4_ROOTFS directory)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: unable to create $MP_4_ROOTFS directory" > /dev/console
	update_error
	exit 1
fi

# Mount new partition
mount -t ext3 /dev/$NewBSPPart $MP_4_PARTFS >/dev/null 2>&1
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot mount /dev/$NewBSPPart on $MP_4_PARTFS)" >> $HBOX_UPDATE_LOG_FILE
	#echo "$EXEC_NAME: unable to mount /dev/$NewBSPPart on $MP_4_PARTFS" > /dev/console
	update_error
	exit 1
fi

# Mount HBoxFirmware file
mount -t ext2 -o loop $ROOTFS_EXT2_PATH $MP_4_ROOTFS > /dev/null 2>&1
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot mount $ROOTFS_EXT2_PATH on $MP_4_ROOTFS)" >> $HBOX_UPDATE_LOG_FILE
   #echo "$EXEC_NAME: unable to mount $ROOTFS_EXT2_PATH on $MP_4_ROOTFS" > /dev/console
   update_error
   exit 1
fi

# Copy new files from HBoxFirmware to new partition
cp -af $MP_4_ROOTFS/* $MP_4_PARTFS
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (ERROR during copying filesystem)" >> $HBOX_UPDATE_LOG_FILE
   #echo "$EXEC_NAME: ERROR during copying filesystem" > /dev/console
	umount $MP_4_ROOTFS
	umount $MP_4_PARTFS
   update_error
   exit 1
fi

# sync filesystems and umount mmc
sync
umount $MP_4_PARTFS

# Update boot_script.img if needed
if [ -f $MP_4_ROOTFS/boot/boot_script.img ] ; then
   rm $MP_4_PARTFS/boot/boot_script.img
   sync
   umount $MP_4_PARTFS
   mount -t ext3 /dev/mmcblk0p1 $MP_4_PARTFS >/dev/null 2>&1
   if [ $? -ne 0 ] ; then
      LogDate=`date +%Y/%m/%d-%H:%M:%S`
      echo "$LogDate : Abort on serious error (Cannot mount /dev/mmcblk0p1 on $MP_4_PARTFS)" >> $HBOX_UPDATE_LOG_FILE
      #echo "$EXEC_NAME: unable to mount /dev/mmcblk0p1 on $MP_4_PARTFS" > /dev/console
      update_error
      exit 1
   fi
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Copy boot_script.img on boot_script partition)" >> $HBOX_UPDATE_LOG_FILE
   #echo "$EXEC_NAME: copy boot_script.img on boot_script partition" > /dev/console
   cp $MP_4_ROOTFS/boot/boot_script.img $MP_4_PARTFS
   if [ $? -ne 0 ] ; then
      LogDate=`date +%Y/%m/%d-%H:%M:%S`
      echo "$LogDate : Abort on serious error (ERROR during copying boot_script.img)" >> $HBOX_UPDATE_LOG_FILE
      update_error
      exit 1
   fi
fi

# sync filesystems
sync

# umount filesystem
umount $MP_4_ROOTFS
umount $MP_4_PARTFS

# Set partition flag
set_partition_as_update $PartNum
if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot set partition \"system_$PartNum\" as \"UPDATE\"" >> $HBOX_UPDATE_LOG_FILE
	update_error
	exit 1
fi

# Update other partition as BACKUP
if [[ $PartNum -eq 1 && $PART_SYS2_IS_VALID -eq 1 ]] ; then
	set_partition_as_backup 2
	if [ $? -ne 0 ] ; then
   LogDate=`date +%Y/%m/%d-%H:%M:%S`
   echo "$LogDate : Abort on serious error (Cannot set partition \"system_2\" as \"BACKUP\"" >> $HBOX_UPDATE_LOG_FILE
		update_error
		exit 1
	fi
elif [[ $PartNum -eq 2 && $PART_SYS1_IS_VALID -eq 1 ]] ; then
	set_partition_as_backup 1
	if [ $? -ne 0 ] ; then
      LogDate=`date +%Y/%m/%d-%H:%M:%S`
      echo "$LogDate : Abort on serious error (Cannot set partition \"system_1\" as \"BACKUP\"" >> $HBOX_UPDATE_LOG_FILE
		update_error
		exit 1
	fi
fi

# All is done,
# Create file for udev rules
echo 1 > /tmp/BSP_UPDATE_OK
update_ok
LogDate=`date +%Y/%m/%d-%H:%M:%S`
echo "$LogDate : UPDATE DONE WITH SUCCESS. (Wait for reboot)" >> $HBOX_UPDATE_LOG_FILE
exit 0
