#!/bin/sh

#########################################################################################
# Hager application automatic starting
#
# Versions
#
# 0.0.1
# Preliminary version that doesn't test the KNX board at all.
#
# 0.0.2
# Add the detection of the ttymxc2 process, to be able to kill it and therefore to allow
# an analyze of the destination file which contain the marks of the dialog with the KNX
# board.
#
# 0.0.3
# Modification of ttyUSB0 initialization to suppress remote echo
#
# 0.0.4
# Modification of detection for the ttyUSB0 redirection as the field returned was not the
# good one (2 instead of 1 previously)
#
# 0.0.5
# Add of STOP_TEST command to stop script execution
#
# 0.0.6
# Kill the ttyUSB0 redirection before leaving the script
#
# 0.0.7
# Change method to retrieve pid of redirection processes as awk is definitely more
# adapted than cut for this purpose.
#
# 0.0.8
# Change method to keep product type using fw_setenv and fw_printenv instead of a local file
#
# 0.0.9
# Change product type value stored from a litteral to a pseudo-numeric value which allow
# combination between values
#
# 0.0.A
# Add an answer to the STOP_TEST command and sent it before ending the script
#
# 1.0.0
# Implements the get KNX board version algorithm
#
# 1.0.1
# Add output levels to display strings depending on the context
#
# 1.0.2
# The script gets its own version and don't share the SCRIPT_VERSION variable anymore
#
# 1.1.0
# Add WRITE_DATE and READ_DATE commands for setting the RTC or reading the system clock
# Modify WRITE_SOFTLIMITATION_STR which become WRITE_SOFTLIMITATION in order to suppress
# useless colon mark
# Modify Output function to manage display depending on a global debugLevel
# Change command decoding algortihm in order to manage commands with variable number
# of parameters
#
# 1.1.1
# Modification of message sent in case of bad time argument in WRITE_DATE command.
# Return was "BAD_DATE_ANSWER" and is now "BAD_TIME_ANSWER"
#
# 1.2.0
# Support EMMS version by setting the correct SOFT_VERSION_FILE according to the board type
#
# 1.2.1
# Implements TEST_SD command with µSD card availability and Dummy File transfer
# Don't try to get KnxBoard software version for HBS hardware
#
# 1.3.0
# Support for nomos
# WRITE_HASHFILE command
#
# 1.3.1
# bug in SetHashfile
#
##########################################################################################

SERIAL_USB_SCRIPT_VERSION=1.3.1

#########################################################################################
# Local variables defined
#########################################################################################
USB_TTY=/dev/ttyUSB0
KNX_TTY=/dev/ttymxc2
SD_CARD_DEV=/dev/mmcblk1p1
SD_CARD_PATH=/media/µSD
DUMMY_FILE=test.bin
MY_TTY=/tmp/myTTYUSB0
MY_KNX=/tmp/myKNX
ACK_KNX="E5"
ANSWER_FILE="/tmp/LastAnswer"
EMPTY_STR=""

##### Product files used
SERIAL_NUMBER_FILE="/proc/HBox-SerialNumber"
BOARD_TYPE_FILE="/proc/HBox_BoardType"
FIRMWARE_VERSION_FILE="/proc/HBox-Firmware-version"
CPU_REVISION_FILE="/proc/HBox-CPURevision"

##### Software files used
SOFT_VERSION_STR="com.hager.application.version"
SOFT_VERSION_FILE_EMSS="/root/app/osgi/bin/vms/emss.prs"
SOFT_VERSION_FILE_KNX="/root/app/osgi/bin/vms/knxng.prs"
SOFT_VERSION_FILE=""

##### GPIOs
GPIO_PATH="/sys/class/gpio"
COM_GPIO="$GPIO_PATH/gpio234"
SEL1_GPIO="$GPIO_PATH/gpio235"
BOARD_ID0_GPIO="$GPIO_PATH/gpio26"
BOARD_ID1_GPIO="$GPIO_PATH/gpio21"

##### Kind of hardware : $BOARD1$BOARD0
HARD_KNX="00"
HARD_HBS="01"
HARD_ELCOM="10"
HARD_UNDEF="11"

##### Commands that can be executed
WRITE_SOFTLIMITATION="WRITE_SOFTLIMITATION"
GET_SCRIPT_VERSION="READ_SCRIPT_VERSION"
GET_SN="READ_SN"
GET_SOFTVERSIONS="READ_VERSIONSOFT"
GET_COMSEL="READ_SELECTORS"
GET_TYPEPRODUCT="READ_SOFTLIMITATION"
STOP_TEST="STOP_TEST"
WRITE_DATE="WRITE_DATE"
GET_DATE="READ_DATE"
TEST_SD_CARD="TEST_SD_CARD"
WRITE_HASHFILE="WRITE_HASHFILE"

##### Command parameters
GREETINGS="Ready for commands"
CONFIGURATOR_STR="CONFIGURATOR"
SERVERDOMOVEA_STR="SERVERDOMOVEA"
NOMOS_STR="NOMOS"
PRODUCT_TYPE_STR="HagerProduct"
PRODUCT_ANSWER_STR="$PRODUCT_TYPE_STR="
PRODUCT_TYPE_CONFIGURATOR="0001"
PRODUCT_TYPE_SERVERDOMOVEA="0002"
PRODUCT_TYPE_NOMOS="0004"
PRODUCT_TYPE_UNKNOWN="Unknown product"
UNKNOWN_PRODUCT="Unknown product"
BACK_FACTORY="BACKFACTORY"
ENDING_TEST="TEST ENDING"

##### Command answers
DATETIME_BAD_PARAM_ANSWER="ERROR - Missing part in Date Time parameters"
BAD_DATE_ANSWER="ERROR - Date not correct"
BAD_TIME_ANSWER="ERROR - Time not correct"
UNKNOWN_COMMAND="Unknown command"
OK_ANSWER="OK"
SD_CARD_OK="µSD transfer OK"
SD_CARD_NOK="µSD transfer NOK"
NO_SD_CARD="µSD card not found"

##### KNX Board commands
KNX_FT12_RESET="\x10\x40\x40\x16"
KNX_BOARD_RESET="\x68\x05\x05\x68\x53\x10\x00\xfd\x06\x66\x16"
KNX_CHECK_ALIVE="\x68\x05\x05\x68\x53\x10\x00\xfd\x04\x64\x16"
KNX_BOARD_VERSION="\x68\x06\x06\x68\x73\x10\x00\xFD\x00\x01\x81\x16"
KNX_ACK="\xe5"
GOOD_ANSWER_EVEN="68 3e 3e 68 d3 2d 00 fd 01"
GOOD_ANSWER_ODD="68 3e 3e 68 f3 2d 00 fd 01"
VERSION_LENGTH=8

##### Outout variable and levels
DBG_OUTPUT="/dev/console"
DEBUG="DEBUG"
DEBUG_WARNING=1
DEBUG_NORMAL=2

#########################################################################################
# Global variables
#########################################################################################
debugLevel=$DEBUG_NORMAL

#########################################################################################
# Fonctions
#########################################################################################
Output() {
	if [ -z "$1" ]; then
		return
	fi
	if [ -z $2 ] || [ $2 -ge $debugLevel ]; then
		TimeElapsed=`awk '{printf("[%11f]", $1)}' /proc/uptime)`
		echo -e "$TimeElapsed $1" >> $DBG_OUTPUT
	fi
}

#########################################################################################
# Function that the test presence of USB/Serial converter.
# First version only test the presence of the cable, further versions must identify the
# vendor ID to be sure that it's the test bench that is connected
#
# Return
# O if detected / 1 otherwise
USBttyPresent() {
	if [ -e $USB_TTY ]; then
		return 0
	else
		return 1
	fi
}

#########################################################################################
# Function that test the presence of KNX Board by looking at the /dev entry
#
# Return
# 0 if detected / 1 otherwise
KNXPresent() {
	if [ -e $KNX_TTY ]; then
		return 0
	else
		return 1
	fi
}

#########################################################################################
# Function that read some gpios which are supposed to contain the base card type
#
# Return
# O if hardware recognized / 1 otherwise
GetProductHardware() {
	hardwareType=""
	BOARD0=""
	BOARD1=""
	if [ -f "$BOARD_ID0_GPIO/value" ] && [ -f "$BOARD_ID0_GPIO/value" ]; then
		BOARD0=$(cat "$BOARD_ID0_GPIO/value")
		BOARD1=$(cat "$BOARD_ID1_GPIO/value")
		hardwareType=$(echo "$BOARD1$BOARD0")
	fi
	if [ ! "$hardwareType" == "$HARD_KNX" ] && [ ! "$hardwareType" == "$HARD_HBS" ] && [ ! "$hardwareType" == "$HARD_ELCOM" ]; then
		return 1
	else
		return 0
	fi
}

#########################################################################################
# Function that return the content of the gpios relating to the switches depending on
# hardware we are running on
GetSwitchesValues() {
	case "$hardwareType" in
		$HARD_KNX | $HARD_ELCOM)
			COM=$(cat "$COM_GPIO/value")
			SEL=$(cat "$SEL1_GPIO/value")
			answer=$(echo "$COM;$SEL")
			;;
		$HARD_HBS)
			COM=$(cat "$COM_GPIO/value")
			answer=$(echo "$COM")
			;;
	esac
}

#########################################################################################
# Function that read the box serial number by using a system file
GetSerialNumber() {
	answer=$(cat $SERIAL_NUMBER_FILE)
}

#########################################################################################
SetProductType() {
	if [ ! -z "$1" ]; then
		answer=$(echo $OK_ANSWER)
		case "$1" in
			$CONFIGURATOR_STR)
				fw_setenv $PRODUCT_TYPE_STR $PRODUCT_TYPE_CONFIGURATOR
				wanted_recovery=1
				;;
			$NOMOS_STR)
				fw_setenv $PRODUCT_TYPE_STR $PRODUCT_TYPE_NOMOS
				wanted_recovery=2
				;;
			$SERVERDOMOVEA_STR)
				fw_setenv $PRODUCT_TYPE_STR $PRODUCT_TYPE_SERVERDOMOVEA
				;;
			*)
				answer=$UNKNOWN_PRODUCT
				;;
		esac

		# If ProductType changed, we need to flash recovery according to testBench command
		# test if flash is needed or not
		if [ $(fw_printenv -n part_recovery) -ne $wanted_recovery ]; then
			Output "recovery $wanted_recovery will be flashed at next boot"
			# Flashing is done by part_sysx_flag corruption at the next boot.
			# this way, HagerBox restarts itself at the end of recovery procedure on part_recovery
			fw_setenv part_recovery $wanted_recovery
			fw_setenv part_sys1_flag NOK
			fw_setenv part_sys2_flag NOK
		else
			Output "Nothing to do, system allready flashed with this recovery !"
		fi
	else
		answer=$UNKNOWN_COMMAND
	fi
}

GetProductType() {
	product=$(echo $(fw_printenv -n $PRODUCT_TYPE_STR))
	case "$product" in
		$PRODUCT_TYPE_CONFIGURATOR)
			answer=$CONFIGURATOR_STR
			;;
		$PRODUCT_TYPE_NOMOS)
			answer=$NOMOS_STR
			;;
		$PRODUCT_TYPE_SERVERDOMOVEA)
			answer=$SERVERDOMOVEA_STR
			;;
		*)
			answer=$PRODUCT_TYPE_UNKNOWN
			;;
	esac
}

#########################################################################################
CheckDateValidity() {
	year=$(echo $1 | cut -d"-" -f1)
	month=$(echo $1 | cut -d"-" -f2)
	day=$(echo $1 | cut -d"-" -f3)
	Output "Year : $year - Month : $month - Day : $day" $DEBUG_WARNING
	if [ -z $year ] || [ -z $month ] || [ -z $day ]; then
		Output "Bad date format (YYYY-MM-DD)"
		return 1
	fi
	## Check year validity
	if [ $year -lt 2015 ] || [ $year -gt 2100 ]; then
		Output "Invalid year : $year"
		return 1
	fi
	## Check month validity
	if [ $month -lt 1 ] || [ $month -gt 12 ]; then
		Output "Invalid month : $month"
		return 1
	fi
	## Check day validity
	if [ $day -lt 1 ] || [ $day -gt 31 ]; then
		Output "Invalid day : $day"
		return 1
	fi
	case "$month" in
		"04" | "06" | "09" | "11")
			if [ $day -gt 30 ]; then
				Output "Invalid day : $day for month $month"
				return 1
			fi
			;;
		"02")
			if [ $day -gt 29 ]; then
				Output "Invalid day : $day for month $month"
				return 1
			fi
			if [ $day -gt 28 ]; then
				## 29 days in February available only on leap year
				leapyear=false
				if [ `expr "$year" % 4` -eq 0 ]; then
					if [ `expr $year % 100` -eq 0 ]; then
						if [ `expr $year % 400` -eq 0 ]; then
							leapyear=true
						fi
					else
						leapyear=true
					fi
				fi
				if [ $leapyear == false ]; then
					Output "Invalid day : $day - Year $year is not a leap year"
					return 1
				fi
			fi
			;;
	esac
	return 0
}

#########################################################################################
CheckTimeValidity() {
	hour=$(echo $1 | cut -d ":" -f1)
	min=$(echo $1 | cut -d ":" -f2)
	sec=$(echo $1 | cut -d ":" -f3)
	Output "Hour : $hour - Minutes : $min - Seconds : $sec" $DEBUG_WARNING
	if [ -z $hour ] || [ -z $min ] || [ -z $sec ]; then
		Output "Bad time format (hh:mm:ss)"
		return 1
	fi
	if [ $hour -lt 1 ] || [ $hour -gt 24 ]; then
		echo "Invalid hour : $hour"
		return 1
	fi
	if [ $min -lt 0 ] || [ $min -gt 59 ]; then
		Output "Invalid minutes : $min"
		return 1
	fi
	if [ $sec -lt 0 ] || [ $sec -gt 59 ]; then
		Output "Invalid seconds : $sec"
		return 1
	fi
	return 0
}

#########################################################################################
SetDate() {
	if [ -z "$1" ]; then
		return 1
	fi
	newdate=$(echo "$1" | cut -d" " -f1)
	newtime=$(echo "$1" | cut -d" " -f2)
	Output "date = $newdate & time = $newtime" $DEBUG_WARNING
	if [ -z $newdate ] || [ -z $newtime ]; then
		answer="$DATETIME_BAD_PARAM_ANSWER"
		return 1
	fi
	if ! CheckDateValidity $newdate; then
		answer="$BAD_DATE_ANSWER"
	fi
	if ! CheckTimeValidity $newtime; then
		answer="$BAD_TIME_ANSWER"
	fi
	## Set system and hardware (RTC) clocks
	date -s "$newdate $newtime" > /dev/null
	hwclock -wu > /dev/null
	answer="$OK_ANSWER"
}

#########################################################################################
GetDate() {
	answer=$(date +"%Y-%m-%d %H:%M:%S")
}

#########################################################################################
TestSdCard() {
	##### is there a µSD card physically connected ?
	if [ ! -e $SD_CARD_DEV ]; then
		answer="$NO_SD_CARD"
	else
		##### Preparing mount point for µSD
		mkdir $SD_CARD_PATH
		mount -o rw $SD_CARD_DEV $SD_CARD_PATH
		##### Generate random file for µSD test transfer
		dd if=/dev/urandom of=/tmp/$DUMMY_FILE bs=1k count=1000
		##### get checksum for this file
		md5sum /tmp/$DUMMY_FILE | sed 's/tmp/media\/µSD/' > /tmp/sum.txt
		##### copy to µSD device
		cp /tmp/$DUMMY_FILE $SD_CARD_PATH
		##### Force flush of all buffers
		sync
		##### check integrity of the copy
		md5sum -c -s /tmp/sum.txt && answer="$SD_CARD_OK" || answer="$SD_CARD_NOK"
		##### clean up and unmount µSD card
		rm $SD_CARD_PATH/$DUMMY_FILE
		umount $SD_CARD_DEV
	fi
	Output "SD card test result : $answer" $DEBUG_NORMAL
	##### some clean up after test
	rm /tmp/$DUMMY_FILE
	rm /tmp/sum.txt
}

#########################################################################################
SetHashfile() {
	if [ ! -z "$2" ]; then
		mkdir -p /tmp/hash
		echo $1 > /tmp/hashfile
		echo $2 " /tmp/hashfile" > /tmp/hash/md5sum.txt
		answer=$(md5sum -c /tmp/hash/md5sum.txt | sed 's/.* //' | sed 's/FAILED/ERROR - Hashfile corrupted/')
		rm -r /tmp/hash
		fw_setenv "hashfile" $1
		fw_printenv -n hashfile | sed 's/#/\n/g' > /tmp/hashfile
	else
		answer="ERROR - Bad parameters"
	fi
}

#########################################################################################
# Function that retrieve the PID of the ttymxc2 redirection in order to kill it.
# Once closed, the destination file is also closed and therefore can be analyzed
KillKNXRedirection() {
#	psfound=$(ps | grep -v -e $0 -e grep | grep "$KNX_TTY" | cut -d ' ' -f 2)
	psfound=$(ps | grep -v -e $0 -e grep | grep "$KNX_TTY" | awk '{print $1}')
	Output "pid avec ps : $psfound" $DEBUG_WARNING
	nbPID=$(echo "$psfound" | wc -w)
	##### Just verify that we found only one PID with these criteria before killing it
	if [ $nbPID == "1" ]; then
		kill -9 $psfound
	fi
}

#########################################################################################
# Function that retrieve the PID of the ttyUSB0 redirection in order to kill it.
KillUSB0Redirection() {
#	psfound=$(ps | grep -v -e $0 -e grep | grep "$USB_TTY" | cut -d ' ' -f 2)
	psfound=$(ps | grep -v -e $0 -e grep | grep "$USB_TTY" | awk '{print $1}')
	Output "pid avec ps : $psfound" $DEBUG_WARNING
	nbPID=$(echo "$psfound" | wc -w)
	##### Just verify that we found only one PID with these criteria before killing it
	if [ $nbPID == "1" ]; then
		kill -9 $psfound
	fi
}

#########################################################################################
# Function that fill a string with the firmware version. obtained from a system file
#
# Return
# 0 if successfull / 1 otherwise
GetBSPVersion() {
	if [ ! -f "$FIRMWARE_VERSION_FILE" ]; then
		bspversion="$FIRMWARE_VERSION_FILE not found"
	else
		bspversion=$(cat "$FIRMWARE_VERSION_FILE")
	fi
	Output "$bspversion" $DEBUG_WARNING
}

#########################################################################################
# Function that fill a string with the software version obtained by reading a specific
# string in a specific file inside software environment
#
# Return
# 0 if successfull / 1 otherwise
GetSoftVersion() {
	case "$hardwareType" in
		$HARD_KNX)
			SOFT_VERSION_FILE=$SOFT_VERSION_FILE_KNX
			;;
		$HARD_HBS)
			SOFT_VERSION_FILE=$SOFT_VERSION_FILE_EMSS
			;;
	esac
	if [ ! -f "$SOFT_VERSION_FILE" ]; then
		softversion="$SOFT_VERSION_FILE not found"
	else
		softversion=$(grep $SOFT_VERSION_STR $SOFT_VERSION_FILE | cut -d = -f 2)
	fi
	Output "$softversion" $DEBUG_WARNING
}

#########################################################################################
# Function that initialize serial port between KNX and CPU boards to : 9600 8E1
InitKNXPort() {
	# stty -F $KNX_TTY 10:0:9bd:0:3:1c:7f:15:4:14:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0	# time 20 / min 1
	stty -F $KNX_TTY 10:0:9bd:0:3:1c:7f:15:4:34:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0	# time 52 / min 1. Parity checking enable
}

#########################################################################################
# Function that verify the validity of KNX board version
#
# Argument
# $1 KNX string version on 3 digits (XX XX XX)
#
# Return
# 0 if successfull / 1 otherwise
VerifyKNXVersionValidity() {
	if [ ! -z "$1" ]; then
		first=$(echo $version | cut -f 1 -d ' ')
		second=$(echo $version | cut -f 2 -d ' ')
		third=$(echo $version | cut -f 3 -d ' ')
		if [ "$first" -ne 1 ]; then
			return 1
		fi
		if [ "$second" -ne 0 ]; then
			return 1
		fi
		if [ "$third" -lt 5 ]; then
			return 1
		fi
		if [ "$third" -gt 96 ]; then
			return 1
		fi
		return 0
	else
		return 1
	fi
}

#########################################################################################
# Function that extract KNX board version from a string given on entry
#
# Arguments
# $1 String which should contain the KNX board version
# $2 Pattern to find to retrieve version
#
# Return
# 0 if successfull / 1 otherwise
ExtractKNXVersion() {
	Output "Trying to extract since : $2\nfrom  :\n$1\n" $DEBUG_WARNING
	source=$1 																													# String with pattern expected
	tofind=$2 																													# Pattern to find
	# Get length of datas before answer pattern
	before="${source%%${tofind}*}"
	Output "substring before pattern: $before" $DEBUG_WARNING
	startingposition=$((${#before}+1))
	Output "pattern found at position : $startingposition" $DEBUG_WARNING
	# Pattern found, get now the version position
	versionposition=$(($startingposition+${#tofind}))
	Output "version should be at position : $versionposition" $DEBUG_WARNING
	# Extract the version part of the string
	version=${source:${versionposition}:${VERSION_LENGTH}}
	OutPut "Version string extracted : $version" $DEBUG_WARNING
	# Verify its validity
	if VerifyKNXVersionValidity $version; then
		knxversion=$(echo $version | awk ' { printf("%d.%d.%d\n", $1, $2, $3) }')
		return 0
	else
		knxversion="KNX board NOT OK !!"
		return 1
	fi
}

#########################################################################################
AnalyzeKNXFile() {
	# Verify that $KNX_TYY redirection produced some datas
	if [ ! -s "$MY_KNX" ]; then																									# If empty file
		knxversion="KNX board NOT OK !!"						# Fill knxversion with a specific message
		return 1 																												# And return an error
	fi
	# Some datas to analyze
	Output "$MY_KNX File content" $DEBUG_WARNING
	Output $(cat $MY_KNX) $DEBUG_WARNING
	# Looks first to an answer with odd parity
	Output "------------------ Looking for odd frame : $GOOD_ANSWER_ODD" $DEBUG_WARNING
	result=$(grep "$GOOD_ANSWER_ODD" "$MY_KNX")
	if [ ! -z "$result" ]; then
		# Answer found, try to use it
		if ExtractKNXVersion "$result" "$GOOD_ANSWER_ODD"; then				# If version extracted
			return 0 								# Return success
		fi
	fi
	# Answer not found, therefore look for an answer with even parity
	Output "------------------ Looking for even frame : $GOOD_ANSWER_EVEN" $DEBUG_WARNING
	result=$(grep "$GOOD_ANSWER_EVEN" "$MY_KNX")
	if [ ! -z "$result" ]; then
		if ExtractKNXVersion "$result" "$GOOD_ANSWER_EVEN"; then			# If version correctly extracted
			return 0 								# Return success
		fi
	fi
	knxversion="KNX board NOT OK !!"
	return 1
}

#########################################################################################
FT12_Reset() {
	printf "\x10\x40\x40\x16" > $KNX_TTY
	usleep 100000							# 0.1 second
}

KNX_CheckAlive() {
	printf "\x68\x05\x05\x68\x53\x10\x00\xfd\x04\x64\x16" > $KNX_TTY
	usleep 100000							# 0.1 second
}

KNX_AskVersion() {
	printf "\x68\x06\x06\x68\x73\x10\x00\xFD\x00\x01\x81\x16" > $KNX_TTY
	usleep 100000
}

FT12_StepForward() {
	printf "\xe5" > $KNX_TTY
	usleep 100000							# 0.1 second
}

#########################################################################################
# Function that does all the dialog with the KNX board.
#
GetKNXVersion() {
	MAX_RETRY=2
	sortie=false
	nbretry=0
	if ! KNXPresent; then
		knxversion="$KNX_TTY not detected"
	else
		##### Set KNX serial port
		InitKNXPort
		while [ $sortie == false ]; do
			if [ ! -e "$MY_KNX" ]; then
				rm $MY_KNX
			fi
			# Lancement en parallèle de la lecture du lien série
			cat $KNX_TTY | od -An -t x1 -v -w1024 > $MY_KNX &
			sleep 1 						# To let enough time for the port to establish correctly
			FT12_Reset
			FT12_Reset
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			KNX_CheckAlive
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			KNX_AskVersion
			FT12_StepForward
			FT12_StepForward
			FT12_Reset
			FT12_Reset
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			FT12_StepForward
			sleep 2
			##### Now kill redirection process to be able to analyze file
			KillKNXRedirection
			usleep 50000
			##### Analyze file
			if AnalyzeKNXFile; then
				sortie=true
			else
				nbretry=`expr $nbretry + 1`
				if [ $nbretry -gt $MAX_RETRY ]; then
					sortie=true
				fi
			fi
		done
	fi
	Output "$knxversion" $DEBUG_WARNING
}

#########################################################################################
# Function that create a string with all element below separated by a semi colon
# BSP version / Soft version / KNX board version
#
# Return
# 0 if every element can be retrieved
# 1 if some is unacessible
GetProductVersions() {
	GetBSPVersion
	GetSoftVersion
	if [ "$hardwareType" == "$HARD_KNX" ] || [ "$hardwareType" == "$HARD_ELCOM" ]; then
		GetKNXVersion
		answer=$(printf "%s;%s;%s" "$knxversion" "$bspversion" "$softversion")
	else
		answer=$(printf "%s;%s" "$bspversion" "$softversion")
	fi
	Output "$answer" $DEBUG_WARNING
}

#########################################################################################
SetAnswer() {
	if [ $# -ge 1 ]; then
		echo "$1" > $ANSWER_FILE
	fi
}

GetAnswer() {
	answer=$(cat $ANSWER_FILE)
}

#########################################################################################
# Function that split a string into 2 global variables (command, params) using space as
# a separator. First part is handle as a command while the other part is handled as a
# bunch of parameters that will be managed later.
#
# Input : $1 - whole command received
# Output : None
extractCmdParam() {
	Output "Received $cmdLine" $DEBUG_WARNING
	command=$(echo "$cmdLine" | cut -d" " -f1)
	lgcmd=${#command}
	lgcmd=`expr $lgcmd + 1`																											# sqeeze space separator between command and params
	params=${cmdLine:$lgcmd}
}

#########################################################################################
# Function that looks on datas received on $USB_TTY and executes available commands.
# Execution return in then written in a file to be exploitable by script core part
WhatHappensOnTTYUSB0() {
	Output "Event $3 on $1 file in $2 directory" $DEBUG_WARNING
	cmdLine=`tail -n1 $MY_TTY`
	extractCmdParam $cmdLine
	Output "Command..$command.. - Params..$params.." $DEBUG_WARNING
	case "$command" in
		$GET_SCRIPT_VERSION)
			SetAnswer "$SERIAL_USB_SCRIPT_VERSION"
			;;
		$STOP_TEST)
			SetAnswer "$ENDING_TEST"
			Output "$answer" $DEBUG_WARNING
			;;
		$GET_SN)
			GetSerialNumber
			SetAnswer "$answer"
			;;
		$GET_SOFTVERSIONS)
			GetProductVersions
			SetAnswer "$answer"
			;;
		$GET_COMSEL)
			GetSwitchesValues
			SetAnswer "$answer"
			;;
		$WRITE_SOFTLIMITATION)
			SetProductType $params
			SetAnswer "$answer"
			;;
		$GET_TYPEPRODUCT)
			GetProductType
			SetAnswer "$answer"
			;;
		$WRITE_DATE)
			SetDate "$params"
			SetAnswer "$answer"
			;;
		$GET_DATE)
			GetDate
			SetAnswer "$answer"
			;;
		$TEST_SD_CARD)
			TestSdCard
			SetAnswer "$answer"
			;;
		$WRITE_HASHFILE)
			SetHashfile $params
			SetAnswer "$answer"
			;;
		*)
			SetAnswer "$UNKNOWN_COMMAND"
			;;
	esac
}

#########################################################################################
#########################################################################################
# Script beginning
#########################################################################################
#########################################################################################

##### Set output level
debugLevel=$DEBUG_NORMAL
if [ ! -z "$1" ] && [ "$1" == $DEBUG ]; then
	debugLevel=$DEBUG_WARNING
fi
Output "############################# $(basename "$0") v$SERIAL_USB_SCRIPT_VERSION"
runscript=true

##### Prepare hashfile in /tmp
fw_printenv -n hashfile | sed 's/#/\n/g' > /tmp/hashfile

##### Verify presence of USB/Serial converter
if ! USBttyPresent; then
	Output "No ttyUSB cable present. Abort"
	exit
fi

##### Try to identify what box we are
if ! GetProductHardware; then
	Output "Hardware not recognized. Abort"
	exit
fi

##### Set ttyUSB0 speed to 9600 bauds
#  stty -F $USB_TTY 2502:5:cbd:8a3b:3:1c:7f:15:4:0:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0
stty -F $USB_TTY 2502:5:cbd:8a33:3:1c:7f:15:4:0:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0 	# Without remote echo

##### Redirect ttyUSB0 to a real file
cat $USB_TTY > $MY_TTY &
sleep 1		# To let enough time for the port to establish correctly

##### Send greetings
echo $GREETINGS > $USB_TTY

##### And do the job
while [ $runscript == true ]; do
	inotifywait $INOTIFY_OPT --format '%f %e %w' $MY_TTY | while read file event location	# Start a sleeping survey on some events that may happens in UPDATE_DIR
	do
		SetAnswer $EMPTY_STR
		WhatHappensOnTTYUSB0 $file $location $event		# Function called in a child process, cause of pipe above
	done
	GetAnswer
	echo $answer > $USB_TTY
	if [ "$answer" == "$ENDING_TEST" ]; then
		runscript=false
		usleep 10000
	fi
done

KillUSB0Redirection

