#! /bin/sh
#############################################################################
# Update from a directory (on a USB key for example).
# 
# 2014 Logilin for FDE
# 20/10/2014 : Modified by FDE <a.courrier@fde.fr>
# 14/11/2014 : Modified by HAGER <m.courivaud@hager.fr>
#
# 1.0.0
# Use of common library to log messages
#############################################################################

UPDATE_FIRMWARE_SCRIPT_VERSION="1.0.0"
UPDATE_FIRMWARE_SCRIPT_INITIALS="[UF-]"

source /usr/local/bin/update_common
source /usr/local/bin/BSPUpdateTools

###################################################################################################
# Variables defined for this script
###################################################################################################

HBOX_UPDATE_LOG_FILE=/data/var/log/update

FW_PREFIX=HBoxFirmware
DEFAULT_FW_VERSION=0.0.0

FW_VERSION_FILE="/proc/HBox-Firmware-version"

###################################################################################################
# Script beginning
###################################################################################################
###################################################################################################

LogUpdate "----------------------------- $(basename "$0") v$UPDATE_FIRMWARE_SCRIPT_VERSION - started"
LogUpdate "BSP update process started"

# Look at current firmware version

if [ -f "${FW_VERSION_FILE}" ] ; then

	CURRENT_FW_VERSION=`cat "$FW_VERSION_FILE" | grep '^[0-9]\+\.[0-9]\+\.[0-9]\+$'`

	if [ "${CURRENT_FW_VERSION}" = "" ] ; then
      	LogUpdate "Current version file is empty --> Use default"
		CURRENT_FW_VERSION=${DEFAULT_FW_VERSION}
	fi
else
   	LogUpdate "Current version file doesn't exists --> Use default"
	CURRENT_FW_VERSION=${DEFAULT_FW_VERSION}
fi

LogUpdate "Current firmware version : $CURRENT_FW_VERSION"

# Extract major, minor and releases numbers

CURRENT_FW_MAJOR=`echo $CURRENT_FW_VERSION | awk -F. '{print $1}'`
CURRENT_FW_MINOR=`echo $CURRENT_FW_VERSION | awk -F. '{print $2}'`
CURRENT_FW_RELEASE=`echo $CURRENT_FW_VERSION | awk -F. '{print $3}'`

# Just erase a previous result if some 

if [ -f /tmp/BSP_UPDATE_OK ] ; then
	rm /tmp/BSP_UPDATE_OK
fi

# Search a firmware file in directory content

cd "$WORKING_DIR"

firmware=""

for fw in `ls -r .`
do

	# Check if some files present in directory
	if [ "$fw" = "*" ] ; then
      LogUpdate "Abort on empty directory"
		exit 1
	fi

	# Looking for a specific file pattern / ^ ignore case for prefix! :-)
	if ! echo "$fw" | grep -i "^${FW_PREFIX}-[0-9]\+\.[0-9]\+\.[0-9]\+"'$' > /dev/null
	then
		continue
	fi

	# Found a filename with expected pattern. Extract version number from the pattern 
	fw_version=`echo $fw | awk -F- '{print $2}'`

	fw_major=`echo $fw_version | awk -F. '{print $1}'`
	fw_minor=`echo $fw_version | awk -F. '{print $2}'`
	fw_release=`echo $fw_version | awk -F. '{print $3}'`

	# Compare version so that only a newer version could be accepted
	if [ $fw_major -lt $CURRENT_FW_MAJOR ] ; then
      	LogUpdate "Firmware found is too old ($fw_version < $CURRENT_FW_VERSION)"
		continue
	fi

	if [ $fw_major -gt $CURRENT_FW_MAJOR ] ; then
		firmware="$fw"
		break
	fi
	
	if [ $fw_minor -lt $CURRENT_FW_MINOR ] ; then
      	LogUpdate "Firmware found is too old ($fw_version < $CURRENT_FW_VERSION)"
		continue
	fi

	if [ $fw_minor -gt $CURRENT_FW_MINOR ] ; then
		firmware="$fw"
		break
	fi

	if [ $fw_release -le $CURRENT_FW_RELEASE ] ; then
      	LogUpdate "Firmware found is too old ($fw_version < $CURRENT_FW_VERSION)"
		continue
	fi

	firmware="$fw"
	break

done	

if [ "$firmware" = "" ] ; then 
   	LogUpdate "Abort on no HBoxFirmware file found"
	exit 1; 
else
   LogUpdate "Correct HBoxFirmware file found"
fi

# Set leds to inform that an update is in progress
update_in_progress
LogUpdate "START UPDATE ($CURRENT_FW_VERSION --> $fw_version)"

# Delete old file if some
if [ -f /tmp/rootfs.ext2 ] ; then
	rm -f /tmp/rootfs.ext2
fi

# The checksump is stored in the first 32 bytes of the firmware.
read_sum=`dd if="$firmware" bs=1 count=32`

#read_sum_version=`dd if="$firmware" bs=1 count=32 skip=32`
#comp_sum_version=`echo -n $fw_version | md5sum - | tr -d ' ' | tr -d '-'`

#if [[ ! "$comp_sum_version" = "$read_sum_version" ]] ; then
#   echo "$0: $firmware wrong CRC Version" > /dev/console
#   update_error
#   exit 1
#fi

# Those checksums are contained in an otherwise empty 4K block.
LogUpdate "Extracting file."
dd if="$firmware" of="/tmp/rootfs.ext2" bs=4k skip=1

# Compute the checksum of the resulting tar file.
comp_sum=`md5sum < "/tmp/rootfs.ext2" | dd bs=1 count=32`

# And compare the two sums.
if [ "$comp_sum" = "$read_sum" ]
then
	# Ok: let's go!
   LogUpdate "CRC OK"

	/usr/local/bin/install_firmware.sh
	exit $?
else
   	LogUpdate "Abort on WRONG CRC"

   	update_error
	exit 1
fi

LogUpdate "----------------------------- $(basename "$0") v$UPDATE_FIRMWARE_SCRIPT_VERSION - ended"
