(function ($, ko) {
	var 
		unwrap = ko.utils.unwrapObservable;
	
	ko.bindingHandlers.cssVisibility = {
		update: function(element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
			var 
				value = unwrap(valueAccessor()),
				visibility = "hidden";
			
			if (value) {
				visibility = "visible";
			}
			$(element).css("visibility", visibility);
		}
	};
	
	ko.bindingHandlers.hidden = {
        update: function(element, valueAccessor) {
            var newValueAccessor = function() {
                return !unwrap(valueAccessor());
            };

            ko.bindingHandlers.visible.update(element, newValueAccessor);
        }
    };
	
	ko.bindingHandlers.inlineEdit = {
		init: function (element, valueAccessor) {
			var
				valueAccessorUnwrapped = unwrap(valueAccessor()),
				classesToAdd = valueAccessorUnwrapped.classes,
				i = 0,
				p = document.createElement("p"),
				$p = null,
				input = document.createElement("input"),
				$input = null,
				observable = valueAccessorUnwrapped.value,
				canEdit =  valueAccessorUnwrapped.canEdit,
				hasValidation = valueAccessorUnwrapped.hasValidation,
				isInEditMode = null; // !!! not to be mixed-up with observable.isInEditMode
			
			if (classesToAdd && classesToAdd.length) {
				$p = $(p);
				$input = $(input);
				for (i = 0; i < classesToAdd.length; i++) {
					$p.addClass(classesToAdd[i]);
					$input.addClass(classesToAdd[i]);
				}
			}

			element.appendChild(p);
			element.appendChild(input);
			
			observable.editing = ko.observable(false);
			
			ko.applyBindingsToNode(element, {
				event: { 
					'dblclick': function () { 
					if (canEdit && typeof canEdit() === "boolean" && !canEdit()) {
						return;
					}
					observable.editing(true); 
				}}
			});
			
			if (hasValidation) {
				observable.isInEditMode = ko.computed({
					read: function () {
						if (this.isValid) {
							return !this.isValid() || this.editing();
						} else {
							return this.editing();
						}
					},
					write: function (value) {
						this.editing(value);
					},
					owner: observable
				});
			}
			
			isInEditMode = hasValidation ? observable.isInEditMode : observable.editing;
			
			ko.applyBindingsToNode(p, {
				text: observable,
				hidden: isInEditMode
			});
			
			ko.applyBindingsToNode(input, {
				value: observable,
				visible: isInEditMode,
				hasfocus: isInEditMode
			});
		}
	};
	
	ko.bindingHandlers.inlineSelectEdit = {
		init: function (element, valueAccessor) {
			var
				valueAccessorUnwrapped = unwrap(valueAccessor()),
				observable = valueAccessorUnwrapped.value,
				options = valueAccessorUnwrapped.options,
				optionsText = valueAccessorUnwrapped.optionsText,
				classesToAdd = valueAccessorUnwrapped.classes,
				p = document.createElement("p"),
				select = document.createElement("select"),
				$p = $(p),
				$select = $(select);
			
			for (var i = 0; i < classesToAdd.length; i++) {
				$p.addClass(classesToAdd[i]);
				$select.addClass(classesToAdd[i]);
			}
			
			element.appendChild(p);
			element.appendChild(select);
			
			observable.editing = ko.observable(false);
			
			ko.applyBindingsToNode(element, {
				event: { 'dblclick': function () { observable.editing(true); } }
			});
			
			ko.applyBindingsToNode(select, {
				options: options,
				optionsText: optionsText || undefined,
				value: observable,
				visible: observable.editing,
				hasfocus: observable.editing
			});
			
			ko.applyBindingsToNode(p, {
				text: optionsText
					? ko.computed(function () {
						return observable().name;
					})
					: observable,
				hidden: observable.editing
			});
		}
	};
}($, ko))