(function ($, ko, ns) {
	ns.ButtonsMenuViewModel = function (buttons) {
		this.buttonsVMs = ko.observableArray();
		this.pressedButtonVM = ko.observable(null);
		this.defaultButtonVMs = [];
		this._construct.call(this, buttons);
	};

	ns.ButtonsMenuViewModel.prototype = function () {
		var
			// private methods
			
			// invoke with call
			_changePressedButton = function (newPressedButtonVM) {
				this.pressedButtonVM().isPressed(false);
				newPressedButtonVM.isPressed(true);
				this.pressedButtonVM(newPressedButtonVM);
			},
		
			// protected methods
			_construct = function (buttons) {
				var
					i = 0,
					buttonsVMs = this.buttonsVMs(),
					buttonVM = null;
					
				for (i = 0; i < buttons.length; i++) {
					buttonVM = new ns.ButtonMenuTab(buttons[i].text, buttons[i].viewName, buttons[i].isPressed, buttons[i].canPress);
					
					if (buttons[i].isPressed) {
						this.pressedButtonVM(buttonVM);
					}
					
					if (buttons[i].isDefault) {
						this.defaultButtonVMs.push(buttonVM);
					}
					
					buttonsVMs.push(buttonVM);
				}
				this.buttonsVMs.valueHasMutated();
			},
			
			// public methods
			
			setDefaultAsPressed = function () {
			    var i = 0,
					defaultButtonVM = null;
				
				for (i = 0; i < this.defaultButtonVMs.length; i++) {
					if (this.defaultButtonVMs[i].canPress()) {
						defaultButtonVM = this.defaultButtonVMs[i];
						break;
					}
				}
				
				if (defaultButtonVM === null) {
					throw new Error('There must be at least one buttonVM that qualifies as "default".');
				}
				_changePressedButton.call(this, defaultButtonVM);
			},
			
			// public commands
			
			pressButtonCommand = function (vm, event) {
				var 
					sourceElement = event.srcElement || event.originalTarget;
				if (sourceElement.nodeType === 1 && sourceElement.nodeName.toLowerCase() !== "button") {
					return;
				}
				_changePressedButton.call(vm, ko.dataFor(sourceElement));
			};
			
		return {
			_construct: _construct,
			pressButtonCommand: pressButtonCommand,
			setDefaultAsPressed: setDefaultAsPressed
		};
	}();
	
	ns.ButtonMenuTab = function (text, value, isPressed, canPress) {
		this.text = text;		
		this.value = value;
		this.isPressed = ko.observable(isPressed);
		this.canPress = typeof canPress === "function" ? canPress : true;
		this.isDisabled = ko.computed(function () {
			return typeof this.canPress === "function" ? !this.canPress() : false;
		}, this);
	};
}($, ko, window.configTree = window.configTree || {}))