(function ($, ko, ns) {
	ns.TreeNodeViewModel = function (model, parent, options/*onNodeInteraction, indentation, isOpen, isSelected*/) {
		this.model = model;
		this.parent = parent;
		this.onNodeInteraction = options.onNodeInteraction;
		this.nodeName = model.nodeName;
		this.children = ko.observableArray();
		this.properties = model.properties;
		this.isOpen = ko.observable(!!options.isOpen);
		this.isSelected = ko.observable(!!options.isSelected);
		this.indentation = options.indentation;
		this.indentationString = this.indentation.value + this.indentation.unit;
		this.hasChanges = this.model.isDirty;

		if (model.configNodeType === ns.config.nodeTypes.node) {
		    this._setNodeIconClasses(ns.config.iconClasses.treeNode);
		} else if (model.configNodeType === ns.config.nodeTypes.mapProperty) {
		    this._setNodeIconClasses(ns.config.iconClasses.mapPropertyTreeNode);
		} else if (model.configNodeType === ns.config.nodeTypes.arrayProperty) {
		    this._setNodeIconClasses(ns.config.iconClasses.arrayPropertyTreeNode);
		}

		this.nodeIcon = ko.computed(function () {
		    return this.isOpen() ? this.openIconClass : this.closedIconClass;
		}, this);

		this._construct();
	};
	
	ns.TreeNodeViewModel.prototype = function () {
	    var
			// private methods
			_setNodeIconClasses = function (classes) {
			    this.openIconClass = classes.open;
			    this.closedIconClass = classes.closed;
			},

			_loadChildren = function () {
				if (this.children().length > 0) {
					return;
				}
				
				var 
					i = 0,
					childrenFromModel = this.model.children(),
					children = [];
			
				if (this.isOpen() || this.isSelected()) {
				    for (i = 0; i < childrenFromModel.length; i++) {
				        children.push(new ns.TreeNodeViewModel(childrenFromModel[i], this, { onNodeInteraction: this.onNodeInteraction, indentation: this.indentation, isOpen: false, isSelected: false }));
					}
				}
			
				if (children.length > 0) {
					this.children(children);
				}
			},
			
			_construct = function () {
				_loadChildren.call(this);
			},
		
			// public methods
			
			setNodeAsSaved = function () {
				var 
					i = 0,
					children = this.children();
					
				for (i = 0; i < children.length; i++) {
					children[i].setNodeAsSaved();
				}
				
				this.model.dirtyFlag().reset();
			},
			
			openCloseCommand = function (treeNodeVM, event) {
				treeNodeVM.onNodeInteraction.apply(treeNodeVM, arguments);
				if (treeNodeVM.children().length === 0 && treeNodeVM.model.children().length === 0) {
					return;
				}
				
				if (treeNodeVM.isOpen()) {
					treeNodeVM.isOpen(false);	
				} else {
					treeNodeVM.isOpen(true);
					_loadChildren.call(treeNodeVM);
				}
			},
			
			getPath = function () {
				var
					path = "",
					currentNode = this.parent;

				while (currentNode !== null) {
				    path = currentNode.nodeName() + "/" + path;
				    currentNode = currentNode.parent;
				}

				return path;
			},
			
			removeChild = function (vm) {
				this.children.remove(vm);
				if (this.children().length === 0) {
					this.isOpen(false);
				}
			},
			
			remove = function () {
				this.parent.removeChild(this);
				this.model.remove();
			};
			
	    return {
		    _construct: _construct,
		    _setNodeIconClasses: _setNodeIconClasses,
		    setNodeAsSaved: setNodeAsSaved,
			openCloseCommand: openCloseCommand,
			getPath: getPath,
			removeChild: removeChild,
			remove: remove,
			type: "TreeNodeViewModel"
		};
	}();
}($, ko, window.configTree = window.configTree || {}))