(function ($, ko, ns) {
	ns.TreeViewViewModel = function (treeModel, config) {
		var 
			self = this;
	
		this.root = ko.observable(new ns.TreeNodeViewModel(treeModel, null, { onNodeInteraction: config.onNodeInteraction, indentation: config.indentation, isOpen: true, isSelected: true }));
		this.indentation = config.indentation;
		this.selectedNodeVM = ko.observable();
		this.onNodeInteraction = config.onNodeInteraction;
		this._newNodesCounter = 0;
		this._newNodeNameBase = "New_Node";
		this.selectNodeCommand = function (treeNodeVM) {
			self.onNodeInteraction.apply(treeNodeVM, arguments);
			if (treeNodeVM === self.selectedNodeVM()) {
				return;
			}
			
			if (typeof self.selectedNodeVM() !== "undefined") {
				self.selectedNodeVM().isSelected(false);
			}
			treeNodeVM.isSelected(true);
			self.selectedNodeVM(treeNodeVM);
		};
		this.selectNodeCommand(this.root());
		this.addTreeNodeCommand = ko.command({
		    execute: function (vm) {
		        vm.addTreeNode(ns.config.nodeTypes.node);
		    },
		    canExecute: function () {
		        var selectedTreeNodeType = this.selectedNodeVM().model.configNodeType;

		        if (selectedTreeNodeType !== ns.config.nodeTypes.node) {
		            return false;
		        }

		        return true;
		    }.bind(this)
		});
		this.addMapPropertyTreeNodeCommand = ko.command({
		    execute: function (vm) {
		        vm.addTreeNode(ns.config.nodeTypes.mapProperty);
		    }
		});
		this.addArrayPropertyTreeNodeCommand = ko.command({
		    execute: function (vm) {
		        vm.addTreeNode(ns.config.nodeTypes.arrayProperty);
		    }
		});
	};
	
	ns.TreeViewViewModel.prototype = function () {
	    var
            treeActionsIconClasses = {
                addTreeNode: ns.config.iconClasses.treeNode.add,
                addMapPropertyTreeNode: ns.config.iconClasses.mapPropertyTreeNode.add,
                addArrayPropertyTreeNode: ns.config.iconClasses.arrayPropertyTreeNode.add
            };
			// private methods
			
			_clearRoot = function (rootVM) {
				rootVM.children.removeAll();
				rootVM.model.children.removeAll();
				rootVM.model.childrenTokens.removeAll();
				// rootVM.properties.removeAll(); // WARNING: if this is uncommented the root configuration will be deleted as well
			},
			
			_updateOnDelete = function (deletedNodeVM, ajaxData) {
				var 
					children = null, // the children among which is the node to be deleted
					index = null,  // index of the deletedNodeVM in its parent children array
					newSelectedNode = null;
	
				if (deletedNodeVM.parent !== null) { // if it has paren check if it has children except the one to delete
					children = deletedNodeVM.parent.children();
					index = deletedNodeVM.parent.children.indexOf(deletedNodeVM);
					
					if (children.length > 1 && index >= 0) {
						if (index === 0) {
							newSelectedNode = children[index + 1];
						} else {
							newSelectedNode = children[index - 1];
						}
					} else if (children.length > 0 && index < 0) {
						newSelectedNode = children[0];
					} else {
						newSelectedNode = deletedNodeVM.parent;
					}
					
					deletedNodeVM.remove();
				} else {
					_clearRoot(deletedNodeVM);
					newSelectedNode = deletedNodeVM;
				}
				
				newSelectedNode.isSelected(true);
				this.selectedNodeVM(newSelectedNode);
			},
			
			_saveCallback = function (ajaxData) {
				this.setNodeAsSaved();
			},
			
			// public methods

            addTreeNode = function (nodeType) {
                var
                    parentVM = this.selectedNodeVM(),
                    parentModel = parentVM.model,
                    newNodeModel = new ns.ConfigNodeModel(null, parentModel, this._newNodeNameBase + "_" + this._newNodesCounter, { isDirty: true, type: nodeType }),
                    newNodeVM = new ns.TreeNodeViewModel(newNodeModel, parentVM, { onNodeInteraction: this.onNodeInteraction, indentation: this.indentation, isOpen: false, isSelected: true }); //model, parent, onNodeInteraction, indentation, isOpen, isSelected, isNodeNew

                this._newNodesCounter++;
                parentVM.children.push(newNodeVM);
                this.selectNodeCommand(newNodeVM);
                parentVM.isOpen(true);
            },

			removeSelectedNodeCommand = function (vm, event) {
		        _updateOnDelete.call(vm, vm.selectedNodeVM());
		    },
			
			saveCommand = ko.command({
			    execute: function (vm) {
			        var
				        data = {},
				        selected = vm.selectedNodeVM(),
				        path = null,
				        index = 0;

			        if (selected.parent !== null) {

			            while (selected.parent.model.configNodeType !== ns.config.nodeTypes.node) {
			                selected = selected.parent;
			            }

			            if (selected.parent.hasChanges()) {
			                dialogs.notification.open("To save this node you have to save its parent.", dialogs.notification.types.warning, "Save Aborted");
			                return;
			            }

			            path = selected.getPath();
			            index = path.indexOf("/");
			            data.path = path.substring(index + 1);
			        }
				    
			       

			        data.type = vm.root().nodeName();
			        data.node = selected.model.toJSON();
			        ns.dataservice.save(data, _saveCallback.bind(selected), selected.model.configNodeType);
			    }
			}),
			
			_construct = function (root) {
			}
		
		return {
			treeActionsIconClasses: treeActionsIconClasses,
		    addTreeNode: addTreeNode,
			removeSelectedNodeCommand: removeSelectedNodeCommand,
			saveCommand: saveCommand,
			_construct: _construct,
			type: "TreeView"
		};
	}();
}($, ko, window.configTree = window.configTree || {}))