(function ($, ko, ns) {
	ns.vm = function () {
	    var
            subscriptionTopics = ns.config.subscriptionTopics,
			// protected constants
            _configurableTypesBlankText = "--- All ---",

	        // public properties
			configurableTypes = ko.observableArray([_configurableTypesBlankText]),
			selectedConfigurableType = ko.observable(),
			treeViewVM = ko.observable(),
			buttonsMenuVM = new ns.ButtonsMenuViewModel([{
			    id: "Properties",
			    isPressed: false,
			    isDefault: true,
			    text: "Properties",
			    viewName: "configNodePropertiesView",
			    canPress: ko.computed(function () {
			        return !!treeViewVM();
			    })
			}, {
			    id: "ImportConfigNode",
			    isPressed: false,
			    text: "Import Config Node",
			    viewName: "importConfigNodeView",
			    canPress: ko.computed(function () {
			        return !!treeViewVM();
			    })
			}, {
			    id: "ExportConfigNode",
			    isPressed: false,
			    text: "Export Config Node",
			    viewName: "exportConfigNodeView",
			    canPress: ko.computed(function () {
			        return !!treeViewVM();
			    })
			}, {
			    id: "ImportZip",
			    isPressed: false,
			    text: "Import Zip",
			    viewName: "importZipView",
			    canPress: ko.computed(function () {
			        return !treeViewVM();
			    })
			}, {
			    id: "ExportZip",
			    isPressed: true,
			    isDefault: true,
			    text: "Export Zip",
			    viewName: "exportZipView",
			    canPress: ko.computed(function () {
			        return !treeViewVM();
			    })
			}]),
			gridViewVM = ko.observable(new ns.GridViewViewModel()),
			rightPanelView = ko.observable(),
			rightPanelTitle = ko.computed(function () {
			    switch (rightPanelView()) {
			        case "configNodePropertiesView": {
			            return "Node Properties";
			            break;
			        } case "exportZipView": {
			            return "Export All to Zip";
			            break;
			        } case "importZipView": {
			            return "Import All from Zip"
			            break;
			        } case "exportConfigNodeView": {
			            return "Export Configuration";
			            break;
			        } case "importConfigNodeView": {
			            return "Import Configuration";
			            break;
			        }
			    }
			}, ns.vm),
			exportPackVM = ko.observable(ns.exportPackViewModel),
			exportConfigNodeVM = ko.observable(ns.exportConfigNodeViewModel),
			importPackVM = ko.observable(ns.importPackViewModel),
			importConfigNodeVM = ko.observable(ns.importConfigNodeViewModel),

			// private properties
			_configurablesModelsMap = {}, // key; configurable name, value: json responce configurable
			_treeViewsVMsToConfigurableNameMap = {},
            _selectedTreeNodePath = ko.computed(function () {
                var _treeViewVM = treeViewVM();

                if (!_treeViewVM) {
                    return null;
                }

                var selectedNodeVM = _treeViewVM.selectedNodeVM();
                return  selectedNodeVM.getPath() + selectedNodeVM.nodeName();
            }).publishOn(subscriptionTopics.treeViewVM.selectedTreeNodePathChanged, false), // used to publish the selected node path
            _selectedTreeNodeSimpleProperties = ko.computed(function () {
                var _treeViewVM = treeViewVM();
                if (!_treeViewVM) {
                    return ko.observableArray();
                }

                return _treeViewVM.selectedNodeVM().properties;
            }).publishOn(subscriptionTopics.treeViewVM.selectedTreeNodePropertiesChanged, false), // used to publish the selected node simple properties
			// private methods
			_onNodeInteraction = function (configNodeVM, event) {
				rightPanelView("configNodePropertiesView");
			},
			
			_getConfigurableTypesCallback = function (jsonResult) {
				jsonResult.unshift(_configurableTypesBlankText);
				configurableTypes(jsonResult);
			},
			
			_getConfigurableRootCallback = function (json) {
				var 
					selectedConfigurableTypeString = selectedConfigurableType();
				
				if (!(selectedConfigurableTypeString in _treeViewsVMsToConfigurableNameMap)) {
					_treeViewsVMsToConfigurableNameMap[selectedConfigurableTypeString] = new ns.TreeViewViewModel(new ns.ConfigNodeModel(json, null, selectedConfigurableTypeString), {
						isRootOpen: true,
						indentation: {
							value: 16,
							unit: "px"
						},
						selectedRootVM: null,
						onNodeInteraction: function () {}
					});
				}
				
				treeViewVM(_treeViewsVMsToConfigurableNameMap[selectedConfigurableTypeString]);
			}, 
			
			_onSelectedConfigurableTypeChange = function (newValue) {
				if (typeof newValue === "undefined" || 
					typeof newValue === null) {
					return;
				}
				
				if (newValue === _configurableTypesBlankText) {
				    treeViewVM(null);
					buttonsMenuVM.setDefaultAsPressed();
					return;
				} else if (!(newValue in _configurablesModelsMap)) {
					ns.dataservice.getConfigurableRoot(newValue, _getConfigurableRootCallback);
				}
			},	
			
			// public methods
			
			init = function () {
				ns.dataservice.getConfigurableTypes(_getConfigurableTypesCallback);
			},
			
			// public commands
			
			// TODO: refactor when gets async
			refreshCommand = ko.command({
			    execute: function (vm) {
			        ns.dataservice.getConfigurableTypes(_getConfigurableTypesCallback);
				
			        var 
					    type = vm.selectedConfigurableType();
				
			        _configurablesModelsMap = {};
			        _treeViewsVMsToConfigurableNameMap = {};
				
			        if (typeof type !== "string" || type === _configurableTypesBlankText) {
			            vm.treeViewVM(null);
			            return;
			        }
				
			        vm.treeViewVM(null);
			        ns.dataservice.getConfigurableRoot(type, _getConfigurableRootCallback);
			    }
			}),
			
			exportZipCommand = function (vm) {
				vm.rightPanelView("exportZipView");
			},
			
			importZipCommand = function (vm) {
				vm.rightPanelView("importZipView");
			},
			
			exportConfigNodeCommand = function (vm) {
				vm.rightPanelView("exportConfigNodeView");
			},
			
			importConfigNodeCommand = function (vm) {
				vm.importConfigNodeVM().importAsRoot(false);
				vm.rightPanelView("importConfigNodeView");
			};
		
	    //rightPanelView(buttonsMenuVM.pressedButtonVM());
	    exportConfigNodeVM().selectedConfigNodePath.subscribeTo(subscriptionTopics.treeViewVM.selectedTreeNodePathChanged, true, function (path) {
	        if (path === null) {
	            return path;
	        }

	        var
                index = path.indexOf("/");

	        if (index >= 0) {
	            return path.substring(index + 1, path.length) + "/";
	        } else {
	            return null;
	        }
	    });
	    importConfigNodeVM().selectedConfigNodePath.subscribeTo(subscriptionTopics.treeViewVM.selectedTreeNodePathChanged, true, function (path) {
	        if (path === null) {
	            return null;
	        }

	        var
                index = path.indexOf("/");

	        if (index >= 0) {
	            return path.slice(index + 1) + "/";
	        } else {
	            return "";
	        }
	    });
	    gridViewVM().dataSource.subscribeTo(subscriptionTopics.treeViewVM.selectedTreeNodePropertiesChanged, true);
		buttonsMenuVM.pressedButtonVM.publishOn(subscriptionTopics.buttonsMenuVM.pressedButtonVMChanged, false);
		rightPanelView.subscribeTo(subscriptionTopics.buttonsMenuVM.pressedButtonVMChanged, true, function (newValue) { return newValue.value; });
		exportConfigNodeVM().configurableTypesBlankText = _configurableTypesBlankText;
		importConfigNodeVM().configurableTypesBlankText = _configurableTypesBlankText;
				
		treeViewVM.subscribe(function (newTreeViewVM) {
		    buttonsMenuVM.setDefaultAsPressed();
		});

		configurableTypes.subscribe(function (newValue) {
			exportPackVM().configurableTypes(newValue.slice(1, newValue.length));
			importPackVM().configurableTypes(newValue.slice(1, newValue.length));
		});
		
		selectedConfigurableType.subscribe(function (newValue) {
			exportConfigNodeVM().selectedConfigurableType(newValue);
			importConfigNodeVM().selectedConfigurableType(newValue);
			_onSelectedConfigurableTypeChange(newValue);
		});
		
		importConfigNodeVM().refresh = function (frameContent) {
			var 
				jsonResponse = JSON.parse(frameContent);
			
			dialogs.details.open(ns.utils.parseImportResponse([jsonResponse]), "Import Result");
			window.setTimeout(function () {refreshCommand(ns.vm);}, 500);
		};
		
		importPackVM().refresh = function (frameContent) {
			if (typeof frameContent !== "string" || frameContent === "") {
				return;
			}

			var 
				jsonResponse = JSON.parse(frameContent);
				
			dialogs.details.open(ns.utils.parseImportResponse(jsonResponse), "Import Zip Result");
			window.setTimeout(function () {refreshCommand(ns.vm);}, 500);
		};
		
		init(); // all starts from here
		return {
			buttonsMenuVM: buttonsMenuVM,
			type: "configTreeVM",
			configurableTypes: configurableTypes,
			selectedConfigurableType: selectedConfigurableType,
			treeViewVM: treeViewVM,
			gridViewVM: gridViewVM,
			rightPanelView: rightPanelView,
			rightPanelTitle: rightPanelTitle,
			exportPackVM: exportPackVM,
			exportConfigNodeVM: exportConfigNodeVM, 
			importPackVM: importPackVM,
			importConfigNodeVM: importConfigNodeVM,
			refreshCommand: refreshCommand,
			exportZipCommand: exportZipCommand,
			importZipCommand: importZipCommand,
			exportConfigNodeCommand: exportConfigNodeCommand,
			importConfigNodeCommand: importConfigNodeCommand
		};
	}();
	
	ko.applyBindings(ns.vm); // this sets the bindings, all that is needed is built at this point
}($, ko, window.configTree = window.configTree || {}));
//@ sourceURL=dynamicScript.js