(function (ns) {
    ns.Pager = function (items, currentPage, options) {
        options = options || {};

        this.items = ko.isObservable(items) ? items : ko.observable(items);
        this.pageSize = options.pageSize || 10;

        var
            self = this,
            _firstPage = 1,
            _lastPage = ko.computed(function () {
                var l = this.items().length,
                    lastPage = l / this.pageSize;

                if (l % this.pageSize !== 0) {
                    lastPage = Math.floor(lastPage) + 1;
                }

                return lastPage;
            }, this);
    
        this.currentPage = ko.observable(currentPage || _firstPage);
        
        this.displayPageStatus = ko.computed(function () {
            return self.currentPage() + ".." + _lastPage()
        }, this)

        this.pagedItems = ko.computed(function () {
            var
                lastItemOnPage = (this.currentPage() * this.pageSize),
                startItemIndex = lastItemOnPage - this.pageSize,
                items = this.items();
            if (lastItemOnPage > items.length) {
                lastItemOnPage = items.length;
            }

            return (function (array, from, to) {
                    var range = [];

                    if (from < 0) {
                        from = 0;
                    }

                    ++to;

                    for (; from < to; from++) {
                        range.push(array[from]);
                    }

                    return range;
            })(items, startItemIndex, lastItemOnPage - 1);
        }, this);

        this.isVisible = ko.computed(function() {
            var last = _lastPage();
            return !!(last && last !== _firstPage);
        }, this);

        this.reset = function () {
            this.currentPage(_firstPage);
        };

        this.firstPageCommand = ko.command({
            execute: function (vm, event) {
                vm.currentPage(_firstPage);
            },
            canExecute: function () {
                if (self.currentPage() > _firstPage) {
                    return true;
                }

                return false;
            }
        });

        this.prevPageCommand = ko.command({
            execute: function (vm, event) {
                vm.currentPage(vm.currentPage() - 1);
            },
            canExecute: function () {
                if (self.currentPage() > _firstPage) {
                    return true;
                }

                return false;
            }
        });

        this.nextPageCommand = ko.command({
            execute: function (vm, event) {
                vm.currentPage(vm.currentPage() + 1);
            },
            canExecute: function () {
                if (self.currentPage() < _lastPage()) {
                    return true;
                }

                return false;
            }
        });

        this.lastPageCommand = ko.command({
            execute: function (vm, event) {
                vm.currentPage(_lastPage());
            },
            canExecute: function () {
                if (self.currentPage() < _lastPage()) {
                    return true;
                }

                return false;
            }
        });
    
    };
})( window.knx = window.knx || {} );