(function ($, ns) {
    ns.dataservice = function () {
        var
            _requestID = 0,
            _sentRequests = 0,
            _module = "KNXAccess",
            _ajaxOptions = {
                global: true, // if false the default Web Admin Console ajax error handler will not work 
                async: true,
                contentType: 'application/json; charset=utf-8',
                type: 'POST',
                dataType: 'json',
                processData: false,
                cache: false,
                url: window.location.protocol + "//" + window.location.host + "/remote/json-rpc"
            },

            _ajax = function (method, params) {
                var
                    deferred = $.Deferred(),
                    options = $.extend({}, _ajaxOptions, { data: JSON.stringify({ jsonrpc: "2.0", id: _requestID++, method: _module + "/" + method, params: params }) });

                // _wireDeffered(deferred);
                // ko.postbox.publish(ns.config.subscriptionTopics.isBusyChanged, true);
                _sentRequests++;
                $.ajax(options).done(function (data, textStatus, jqXHR) {
                    if (data.error) { // exception which has occured during code execution on the server, NOT to be mistaken with Server Error 5xx
                        deferred.reject(data.error);
                        return;
                    }

                    deferred.resolve(data.result);
                }).fail(function (jqXHR, textStatus, errorThrown) {
                    ns.modalDialog.notification.open(textStatus, errorThrown, ns.modalDialog.notification.types.error);
                }).always(function () {
                    _sentRequests--;
                });

                return deferred.promise();
            },

            readAllFrames = function (driverId) {
                return _ajax("readAllFrames", [driverId]);
            },

            getDataLinkServices = function () {
                return _ajax("getDataLinkServices", []);
            },

            readGroupValue = function (driverId, destination) {
                return _ajax("readGroupValue", [driverId, destination]);
            },
            writeGroupValueLong = function ( driverId, destination, data ) {
                return _ajax("writeGroupValueLong", [driverId, destination, data])
            },
            writeGroupValueShort = function ( driverId, destination, data ) {
                return _ajax("writeGroupValueShort", [driverId, destination, data])
            };

        return {
            readAllFrames: readAllFrames,
            getDataLinkServices: getDataLinkServices,
            readGroupValue: readGroupValue,
            writeGroupValueLong: writeGroupValueLong,
            writeGroupValueShort: writeGroupValueShort
        };
    }();
})(jQuery, window.knx = window.knx || {});