(function (ns) {
    var gridView = function () {
        var items = ko.observableArray(),
            columnHeaderClasses = {notSorted: 'header', descending: 'header headerSortDown', ascending: 'header headerSortUp'},
            columns = [
                new ns.GridViewColumnHeader(i18n.itemTitles.timeTitle, { key: "date", isSorted: true, isAscending: false, iconClasses: columnHeaderClasses}),
                new ns.GridViewColumnHeader(i18n.itemTitles.msgCodeTitle, { key: "message",isSorted: false, isAscending: false, iconClasses: columnHeaderClasses }),
                new ns.GridViewColumnHeader(i18n.itemTitles.sourceTitle, { key: "source",isSorted: false, isAscending: false, iconClasses: columnHeaderClasses }),
                new ns.GridViewColumnHeader(i18n.itemTitles.destinationTitle, { key: "destination",isSorted: false, isAscending: false, iconClasses: columnHeaderClasses }),
                new ns.GridViewColumnHeader(i18n.itemTitles.apciTitle, { key: "apci",isSorted: false, isAscending: false, iconClasses: columnHeaderClasses }),
                new ns.GridViewColumnHeader(i18n.itemTitles.dataTitle, { key: "frameLengthDisplay",isSorted: false, isAscending: false, iconClasses: columnHeaderClasses }),
                new ns.GridViewColumnHeader(i18n.itemTitles.actionsTitle)
            ],
            _lastSortedColumn = null,
            pager = new ns.Pager(items, null, {pageSize: 20}),
            _driverID = ko.computed(function() {
                return ns.vm.driversContainer.driverIdCurrent();
            });

        function _sortItems(key, isAscending) {
            items.sort(function(a, b) {
                var e1 = a[key],
                    e2 = b[key];

                if( e1 > e2 ) {
                    return isAscending ? 1 : -1;
                }

                if( e1 < e2 ) {
                    return isAscending ? -1 : 1;
                }

                return 0;
            });
        };

        function onHeaderClicked(vm, event) {
            var sourceElement = event.source || event.srcElement || event.target,
                sourceVM = ko.dataFor(sourceElement);
                if (sourceVM.isSorted()) {
                    _sortItems(sourceVM.sortKey, sourceVM.isSorted.isAscending());
                    items.reverse();
                    sourceVM.isSorted.isAscending(!sourceVM.isSorted.isAscending());
                } else {
                    _sortItems(sourceVM.sortKey, sourceVM.isSorted.isAscending());
                    _lastSortedColumn.resetSortState();
                    sourceVM.isSorted(true);
                    _lastSortedColumn = sourceVM;
                }
        };

        function _repeatOperation(element) {
            var vm = ko.dataFor(element),
                val;

            if (vm.dataLong) {
                val = vm.dataLong;
            } else {
                val = vm.dataShort;
            }

            ns.vm.onPerformOperationClick(_driverID() || "", vm.destination.toString(), val, 'GROUPVALUE_READ', vm.apci);
        };

        function onActionClicked(vm, event) {
            var $sourceElement = $(event.source || event.srcElement || event.target),
                $parent;

                if ($sourceElement.attr('data-execute') && !$sourceElement.is(":disabled")) {
                    _repeatOperation($sourceElement[0]);
                }

                $parent = $sourceElement.parent();

                if ($parent.attr('data-execute') && !$parent.is(":disabled")) {
                    _repeatOperation($sourceElement.parent()[0]);
                    $parent = null;
                }
        };

        function update(data, append) {
            var logItems = [];
            if(_lastSortedColumn) {
                 _lastSortedColumn = _lastSortedColumn;
            }else _lastSortedColumn = columns[0];

            for(var i = 0; i < data.length; i++) {
                var cemi = data[i].Frame;

                switch(cemi.MessageCode) {
                    case 0x11: //L_Data_req
                    case 0x2E: //L_Data_con
                    case 0x29: //L_Data_ind
                            var source = knxAddr(cemi.Source),
                            destination = knxAddr(cemi.Destination, 2) + " [" + knxAddr(cemi.Destination, 3, '/', 1) + "]",
                            apciString = ns.apciCodeMap[cemi.APCI] ? ns.apciCodeMap[cemi.APCI] : returnDefaultCode2String(cemi.APCI), //acpi2String(cemi.APCI)
                            frameLengthDisplay = cemi.FrameLength > 1 ? textDataLong(cemi.DataLong) : cemi.DataShort
                        break;
                    default: source = "-",
                            destination = "-",
                            apciString = "-",
                            frameLengthDisplay = "-";
                }

                logItems.push({
                    date: data[i].Timestamp,
                    dateDisplay: new Date(data[i].Timestamp).toLocaleTimeString(),
                    message: ns.messageCodeMap[cemi.MessageCode] ? ns.messageCodeMap[cemi.MessageCode] : returnDefaultCode2String(cemi.MessageCode), //messageCode2String(cemi.MessageCode)
                    source: source,
                    destination: cemi.Destination,
                    destinationDisplay: destination,
                    apci: apciString,
                    frameLength: cemi.FrameLength,
                    frameLengthDisplay: frameLengthDisplay,
                    dataLong: cemi.DataLong,
                    dataShort: cemi.DataShort
                });
            };
            items(logItems);
            _sortItems(_lastSortedColumn.sortKey, _lastSortedColumn.isSorted.isAscending())
        };

        return {
            pager: pager,
            itemTitles: i18n.itemTitles,
            onHeaderClicked: onHeaderClicked,
            onActionClicked: onActionClicked,
            columns: columns,
            update: update
        };
    }();
    ns.vm.gridViewViewModel(gridView);
})(window.knx = window.knx || {});