(function (ns) {
    var operationsContainer = function (caption, sortOptions) {
        var _currentDriverId = ko.computed(function (){
                return ns.vm.driversContainer.driverIdCurrent();
            }),
            currentdptType = ko.observable(),
            _dptGroups = i18n.knxOperations.dptgroups,
            _dptValues = i18n.knxOperations.dptvalues,
            valueLabel = i18n.knxOperations.valueLabel,
            valueVal = ko.observable();
            addressLabel = i18n.knxOperations.addressLabel,
            addressVal = ko.observable("1/1/1"),
            performButtonTitle = i18n.knxOperations.performButtonTitle,
            operationTitle = i18n.knxOperations.operationTitle,
            typeLabel = i18n.knxOperations.typeLabel,
            optionGroups = ko.observableArray(),
            tooltips = ns.tooltipMap;

        addressVal.isValid = ko.computed(function() {
            var isValid = false;
            isValid = checkAddress( this() );

            return isValid;
        }, addressVal);

        valueVal.isValid = ko.computed(function() {
            var isValid = false,
                value = this(),
                dptType = currentdptType();
            isValid = checkValue(value, dptType);

            if(typeof isValid !== 'boolean') {

                isValid = true; // checkValue's behavior
            }

            return isValid;
        }, valueVal);

        valueVal.tooltip = ko.observable();

        function Group(label, children) {
            this.label = label;
            this.children = ko.observableArray(children);
        };
        function Option(label, value, tooltip) {
            this.label = label;
            this.value = value;
            this.tooltip = tooltip;
        }

        optionGroups([
            new Group(_dptGroups['read'], [
                new Option(_dptValues['_read'], '_read')
            ]),
            new Group(_dptGroups['simple'], [
                new Option(_dptValues['DPTBoolean'], 'DPTBoolean', tooltips['booleans']),
                new Option(_dptValues['DPT3BitControlled'], 'DPT3BitControlled', tooltips['3bitControlled'])
            ]),
            new Group(_dptGroups['byte'], [
                new Option(_dptValues['DPT8BitSignedValue'], 'DPT8BitSignedValue', tooltips['8bitSigned']),
                new Option(_dptValues['DPT8BitUnsignedValue'], 'DPT8BitUnsignedValue', tooltips['8bitUnsigned']),
                new Option(_dptValues['DPT2ByteSignedValue'], 'DPT2ByteSignedValue', tooltips['2byteSigned']),
                new Option(_dptValues['DPT2ByteUnsignedValue'], 'DPT2ByteUnsignedValue', tooltips['2byteUnsigned']),
                new Option(_dptValues['DPT4ByteSignedValue'], 'DPT4ByteSignedValue', tooltips['4byteSigned']),
                new Option(_dptValues['DPT4ByteUnsignedValue'], 'DPT4ByteUnsignedValue', tooltips['4byteUnsigned'])
            ]),
            new Group(_dptGroups['float'], [
                new Option(_dptValues['DPT2ByteFloatValue'], 'DPT2ByteFloatValue'),
                new Option(_dptValues['DPT4ByteFloatValue'], 'DPT4ByteFloatValue')
            ]),
            new Group(_dptGroups['raw'], [
                new Option(_dptValues['_raw'], '_raw')
            ])

        ]);

        currentdptType(optionGroups()[0].children()[0].value);

        function _onDataTypeChange(vm, event) {
             var sourceElement = event.source || event.srcElement || event.target,
                sourceVM = ko.dataFor(sourceElement),
                currentSelected = ko.dataFor( $(sourceElement).find(":selected")[0] );
                if ($(sourceElement).attr('id')==='dptType') {
                    currentdptType( $(sourceElement).val() );
                    if(currentdptType() === '_read') {
                        valueVal('');
                    }
                    valueVal.tooltip(currentSelected.tooltip);
                }
        };

        function checkAddress( addressVal ) {
            if (/^\d{1,2}\/\d{1,2}\/\d{1,3}$/.test( addressVal ) ||
                /^\d{1,3}\/\d{1,3}$/.test( addressVal ) ||
                /^\d+$/.test( addressVal )) {
                return true;
            } else {
                return false;
            }
        };

        function checkValue( dptValue, dptType ) {
            var isOk = true;
            var ret = false;
            if ('_read' == dptType) {
                ret = true;
                // always ok
            } else if ('_raw' == dptType) {
                ret = [];
                $.each(dptValue.split(','), function(idx, el) {
                    el = parseInt(el);
                    if (isNaN(el)) isOk = false;
                    ret.push(el);
                });
                // TODO:
            } else if ('DPTBoolean' == dptType) {
                // if not OK, value will be 0 (false)
                ret = 'true' == dptValue || 'on' == dptValue || '1' == dptValue ? 1 : 0;
            } else if ('DPT3BitControlled' == dptType) {
                ret = parseInt(dptValue);
          // TODO: 0 <= ret <= 15
                isOk = !isNaN(ret);
            } else if ('DPT2ByteFloatValue' == dptType) {
                var value = new Number(dptValue).valueOf();
                isOk = !isNaN(value);
                if (isOk) {
                    var bits = value * 100;
                    var e = 0;
                    var m = 0;

                    while (value < 0 ? bits < -2048 : bits > 2047) { e++; bits /= 2; }

                    m = Math.round(bits);

                    ret = [0,0];
                    ret[1] = (m & 0x00FF);
                    ret[0] = ((value < 0) ? 0x80 : 0);
                    ret[0] |= (e & 0x0F) << 3;
                    ret[0] |= (m & 0x0700) >> 8;
                }
            } else if ('DPT4ByteFloatValue' == dptType) {
                var value = new Number(dptValue).valueOf();
                isOk = !isNaN(value);
                if (isOk) {
                    var bits = floatToIntBits(value);
                    ret = [0,0,0,0];
                    ret[3] = (bits & 0xff);
                    ret[2] = ((bits >> 8) & 0xff);
                    ret[1] = ((bits >> 16) & 0xff);
                    ret[0] = ((bits >> 24) & 0xff);
                }
            } else if ('DPT8BitSignedValue' == dptType || 'DPT8BitUnsignedValue' == dptType) {
                ret = [ parseInt(dptValue) ];
                isOk = !isNaN(ret[0]);
            } else if ('DPT2ByteSignedValue' == dptType || 'DPT2ByteUnsignedValue' == dptType) {
                var value = parseInt(dptValue);
                isOk = !isNaN(value);
                if (isOk) {
                    ret = [0,0];
                    ret[1] = (value & 0xff);
                    ret[0] = ((value >> 8) & 0xff);
                }
            } else if ('DPT4ByteSignedValue' == dptType || 'DPT4ByteUnsignedValue' == dptType) {
                var bits = parseInt(dptValue);
                isOk = !isNaN(bits);
                if (isOk) {
                    ret = [0,0,0,0];
                    ret[3] = (bits & 0xff);
                    ret[2] = ((bits >> 8) & 0xff);
                    ret[1] = ((bits >> 16) & 0xff);
                    ret[0] = ((bits >> 24) & 0xff);
                }
            } else {
                isOk = false;
            }



            if (!isOk) {
                ret = false;
            }
            return ret;
        };

        var performButton = ko.command({
            execute: function (vm, event) {
            var sourceElement = event.source || event.srcElement || event.target,
                sourceVM = ko.dataFor(sourceElement);

                if ($(sourceElement).attr('id')==="dptExec") {
                    ns.vm.onPerformOperationClick(_currentDriverId(), addressVal(), checkValue(valueVal(), currentdptType()), '_read', currentdptType());
                    return false;
                }
            },
            canExecute: function () {
                if (valueVal.isValid() && addressVal.isValid()) {
                    return true;
                }
                return false;
            }
        });


        return {
            dptTypeChange: _onDataTypeChange,
            optionGroups: optionGroups,
            operationTitle: operationTitle,
            typeLabel: typeLabel,
            valueLabel: valueLabel,
            valueVal: valueVal,
            addressLabel: addressLabel,
            addressVal: addressVal,
            performButtonTitle: performButtonTitle,
            performButton: performButton,
            currentdptType: currentdptType
            // operationToPerform: operationToPerform
        }
    }();
    ns.vm.operationsContainer( operationsContainer );
})( window.knx = window.knx || {} );