﻿
function returnDefaultCode2String(mcode) {
    return "0x" + new Number(mcode).toString(16);
};

function knxAddr(address, format, separator, group) {
	var fmt = format || 3;
	switch(fmt) {
		case 2: // 2 component (group)
			var sep = separator || '/'; // default separator for groups is /, for physical - .
			return ((address & 0x7800) >> 11) + sep + (address & 0x07FF);
		case 3: // 3 component (physical)
			var sep = separator || group ? '/' : '.'; // default separator for groups is /, for physical - .
			return group ?
				((address >>> 11) & 0xff) + sep + ((address >> 8) & 0x07) + sep + (address & 0xff) :
				((address & 0xF000) >> 12) + sep + ((address & 0x0F00) >> 8) + sep + (address & 0xFF);
		default:
			return address;
	}
};

function floatToIntBits(f) {
    var NAN_BITS = 0|0x7FC00000;
    var INF_BITS = 0|0x7F800000;
    var ZERO_BITS = 0|0x00000000;
    var SIGN_MASK = 0|0x80000000;
    var EXP_MASK = 0|0x7F800000;
    var MANT_MASK = 0|0x007FFFFF;
    var MANT_MAX = Math.pow(2.0, 23) - 1.0;

    if (f != f)
        return NAN_BITS;
    var hasSign = f < 0.0 || (f == 0.0 && 1.0 / f < 0);
    var signBits = hasSign ? SIGN_MASK : 0;
    var fabs = Math.abs(f);

    if (fabs == Number.POSITIVE_INFINITY)
        return signBits | INF_BITS;

    var exp = 0, x = fabs;
    while (x >= 2.0 && exp <= 127) {
        exp++;
        x /= 2.0;
    }
    while (x < 1.0 && exp >= -126) {
        exp--;
        x *= 2.0;
    }
    var biasedExp = exp + 127;

    if (biasedExp == 255)
        return signBit | INF_BITS;
    if (biasedExp == 0) {
        var mantissa = x * Math.pow(2.0, 23) / 2.0;
    } else {
        var mantissa = x * Math.pow(2.0, 23) - Math.pow(2.0, 23);
    }

    var expBits = (biasedExp << 23) & EXP_MASK;
    var mantissaBits = mantissa & MANT_MASK;

    return signBits | expBits | mantissaBits;
};

function hexArray(x) {
	var ret = '[ ';
	for(var i in x) {
		var val = x[i] & 0xFF;
		if (val < 16) ret += '0';
		ret += (new Number(val)).toString(16) + ' ';
	}
	return ret.toUpperCase() + ']';
};

function bytesToFloat16(x) {
    var e = ((x[0] & 0x78) >> 3);
    var m = ((x[0] & 0x07) << 8) | x[1] & 0xff;
    if (x[0] & 0x80) m += -2048;
    return (m << e) / 100.0;
}

function bytes2Float32(x) {
	var bytes = (((x[0] & 0xFF) << 24) | ((x[1] & 0xFF) << 16) | ((x[2] & 0xFF) << 8)  | (x[3] & 0xFF));
    var sign = (x[0] & 0x80) ? -1 : 1;
    var exponent = ((bytes >> 23) & 0xFF) - 127;
    var significand = (bytes & ~(-1 << 23));

    if (exponent == 128)
        return sign * ((significand) ? Number.NaN : Number.POSITIVE_INFINITY);

    if (exponent == -127) {
        if (significand == 0) return sign * 0.0;
        exponent = -126;
        significand /= (1 << 22);
    } else {
    	significand = (significand | (1 << 23)) / (1 << 23);
    }
    return sign * significand * Math.pow(2, exponent);
}

function textDateTime(x) {
	var ret = null;
	if ((x[1] & 0xFF) <= 60 && (x[2] & 0xFF) <= 60) {
		var hour = x[0] & 0x1F;
		ret = hour < 10 ? '0' : '';
		ret += hour + ':';
		var min = x[1] & 0x3F;
		if (min < 10) ret += '0';
		ret += min + ':';
		var sec = x[2] & 0x3F;
		if (sec < 10) ret += '0';
		ret += sec;
	}
	if ((x[0] & 0xFF) <= 31 && (x[1] & 0xFF) <= 12 && (x[2] & 0xFF) <= 99) {
		if (ret) ret += ' | ';
		else ret = '';
		var day = x[0] & 0x1F;
		if (day < 10) ret += '0';
		ret += day + '/';
		var month = x[1] & 0x0F;
		if (month < 10) ret += '0';
		ret += month + '/';
		var year = x[2] & 0x7F;
		if (year < 10) ret += '200';
		else ret += (year < 90) ? '20' : '19';
		ret += year;
	}
	return ret;
}

function textDataLong(x) {
	var ret = hexArray(x);
	if (x.length == 2) {
		ret += ' | ' + (((x[0] & 0xFF) << 8) | (x[1] & 0xFF)) + ' | ' + bytesToFloat16(x);
	} else if (x.length == 3) {
		var dateTime = textDateTime(x);
		if (dateTime) ret += ' | ' + dateTime;
	} else if (x.length == 4) {
		ret += ' | ' + (((x[0] & 0xFF) << 24) | ((x[1] & 0xFF) << 16) | ((x[2] & 0xFF) << 8)  | (x[3] & 0xFF)) + ' | ' + bytes2Float32(x);
	}
	return ret;
}


$(function(){

	knx.vm.activate();
	ko.applyBindings(knx.vm, document.getElementById("mainWrapper"));
});