(function (ko, ns) {
    var modalDialogID = "modalDialog",
        modalDialog = function () {
            var
                _queue = [],
                _constructHTML = function () {
                    var
                        div = document.createElement("div"),
                        binding = document.createAttribute("data-bind");

                    div.id = modalDialogID;
                    div.className = "modalDialogCurtain";
                    binding.value = "visible: contentVM(), click: closeCommand, clickBubble: false";
                    div.setAttributeNode(binding);
                    div.innerHTML = '<!-- ko if: contentVM() --><div class="modalDialog ui-corner-all" data-bind="template: { name: contentView, data: contentVM }"></div><!-- /ko -->';
                    document.body.appendChild(div);
                },
                contentVM = ko.observable(null),
                contentView = ko.observable(null),
                _closeCallback = null,
                show = function (vm, view, closeCallback) {
                    //implement queue for showing dialogs
                    _queue.push({ vm: vm, view: view });
                    _closeCallback = closeCallback;
                    contentView(_queue[0].view);
                    contentVM(_queue[0].vm);
                },
                hide = function (vm) {
                    vm = vm || this;

                    if (_closeCallback) {
                        _closeCallback();
                    }

                    vm.contentVM(null);
                    vm.contentView(null);
                    _queue.shift();

                    if (_queue.length > 0) {
                        show(_queue[0].vm, _queue[0].view);
                    }
                },
                closeCommand = function (baseVM, event) {
                    var vm = contentVM();

                    if (vm && vm.closeCommand) {
                        vm.closeCommand.call(vm, vm, event);
                    }
                };
            
            if (!document.getElementById(modalDialogID)) {
                _constructHTML();
            }

            return {
                contentVM: contentVM,
                contentView: contentView,
                closeCommand: closeCommand,
                hide: hide,
                show: show
            };
        }();
    ko.applyBindings(modalDialog, document.getElementById(modalDialogID));

    modalDialog.confirmation = function (modalDialog) {
        var
            _VM = function (title, content, deferred, options) {
                this.title = title;
                this.content = content; // if content is html use html binding, property isHTML
                this.isContentHTML = !!options.isContentHTML; // escaping should be done before passing the content
                this.closeCommand = function (vm) {
                    modalDialog.hide();
                };
                this.confirmCommand = function (vm) {
                    deferred.resolve();
                    modalDialog.hide();
                };
            },
            _view = "confirmDialogView",
            open = function (title, content, options) {
                var deferred = $.Deferred();
                modalDialog.show(new _VM(title, content, deferred, { isContentHTML: !!(options && options.isContentHTML) }), _view, function () { deferred.reject(); });
                return deferred.promise();
            };

        return {
            open: open
        };
    }(modalDialog);

    modalDialog.singleSelection = function (modalDialog) {
        var
            _VM = function (title, items, deferred) {
                this.title = title;
                this.items = items;
                this.closeCommand = function (vm) {
                    modalDialog.hide();
                };
                this.selectCommand = function (vm, event) {
                    var
                        sourceElement = event.srcElement || event.originalTarget || event.target,
                        data = ko.dataFor(sourceElement) || sourceElement.textContent || sourceElement.innerText;

                    deferred.resolve(data);
                    modalDialog.hide();
                };
            },
            _view = "singleSelectDialogView",
            open = function (title, items) {
                var deferred = $.Deferred();
                modalDialog.show(new _VM(title, items, deferred), _view, function () { deferred.reject(); });
                return deferred.promise();
            };

        return {
            open: open
        };
    }(modalDialog);

    modalDialog.notification = function (modalDialog) {
        var
            types = {
                information: 0,
                success: 1,
                warning: 2,
                error: 3
            },
            _types = {
                0: {
                    iconClass: "ui-icon-info",
                    title: "Information"
                },
                1: {
                    iconClass: "ui-icon-circle-check",
                    title: "Success"
                },
                2: {
                    iconClass: "ui-icon-alert",
                    title: "Warning"
                },
                3: {
                    iconClass: "ui-icon-circle-close",
                    title: "Error"
                }
            },
            buttonPositions = {
                left: 0,
                center: 1,
                right: 2
            },
            _buttonPositions = {
                0: "position-left",
                1: "position-center",
                2: "position-right"
            },
            _VM = function (subTitle, content, type, buttonPosition) {
                if (typeof type === "undefined" || type === null) {
                    type = types.information;
                }

                if (typeof buttonPosition === "undefined" || buttonPosition === null) {
                    buttonPosition = buttonPositions.right;
                }

                this.title = _types[type].title;
                this.subTitle = subTitle;
                this.content = content;
                this.iconType = ko.computed(function () {
                    return _types[type].iconClass;
                });
                this.buttonPosition = ko.computed(function () {
                    return _buttonPositions[buttonPosition];
                });
                this.closeCommand = function (vm) {
                    modalDialog.hide();
                };
            },
            _view = "notificationDialogView",
            open = function (subTitle, content, type, buttonPosition) {
                modalDialog.show(new _VM(subTitle, content, type, buttonPosition), _view);
            };

        return {
            open: open,
            types: types,
            buttonPositions: buttonPositions
        };
    }(modalDialog);

    ns.modalDialog = modalDialog;
})(ko, window.knx = window.knx || {});