(function (ns, $) {
	ns.confirmation = function () {
		var 
			$confirmDialog = $("#confirm-dialog"),
			resultCallback = null,
			callbackArguments = null,
			onDialogClose = function () {
				var 
					callback = resultCallback,
					callbackArgs = callbackArguments;
	
				resultCallback = null;
				callbackArguments = null;
				$confirmDialog.html("");
				if (typeof callback === "function") {
					callback.call(this, callbackArgs || {confirmed: false});
				}	
			},
			onOKButtonClick = function () {
				callbackArguments = {confirmed: true};
				$confirmDialog.dialog("close");
			},
			onCloseButtonClick = function () {
				callbackArguments = {confirmed: false};
				$confirmDialog.dialog("close");
			},
			open = function (contentHTML, title, callback) {
				$confirmDialog.html(contentHTML);
				$confirmDialog.dialog("option", "title", title);
				resultCallback = callback;
				$confirmDialog.dialog("open");
			},
			destroy = function () {
				$confirmDialog.unbind();
				$confirmDialog.dialog("destroy");
				$confirmDialog.remove();
			};
		
		if ($confirmDialog.length === 0) {
			$("body").append('<div id="confirm-dialog"></div>');
		}
		
		$confirmDialog.dialog({
			autoOpen: false,
			modal: true,
			resizable: false,
			buttons: {
				Cancel: onCloseButtonClick,
				OK: onOKButtonClick
			}
		});
		
		$confirmDialog.bind("dialogclose", onDialogClose);
		$confirmDialog.addClass("ui-dialogs");
		
		return {
			open: open,
			destroy: destroy,
			$container: $confirmDialog
		};
	}();

	ns.notification = function () {
		var 
			$notificationDialog = $("#notification-dialog"),
			closeCallback = null,
			$icon = null,
			lastIconClass = "",
			$subTitle = null,
			$content = null,
			types = {
				information: 0,
				success: 1,
				warning: 2,
				error: 3 
			},
			_types = {
				0: {
					iconClass: "ui-icon-info",
					title: "Information"
				},
				1: {
					iconClass: "ui-icon-circle-check",
					title: "Success"
				},
				2: {
					iconClass: "ui-icon-alert",
					title: "Warning"
				},
				3: {
					iconClass: "ui-icon-circle-close",
					title: "Error"
				}
			},
			onDialogClose = function () {
				var 
					callback = closeCallback;
		
				closeCallback = null;
				$content.html("");
				if (typeof callback === "function") {
					callback.apply(arguments);
				}	
			},
			onOKButtonClick = function () {
				$notificationDialog.dialog("close");
			},
			open = function (content, type, subTitle, callback) {
				type = typeof type === "number" ? _types[type] : _types[0];	
				$notificationDialog.dialog("option", "title", type.title);
				$content.html(content);
				$icon.removeClass(lastIconClass);
				lastIconClass = type.iconClass;
				$icon.addClass(type.iconClass);
				$subTitle.text(subTitle || "");
				closeCallback = callback;
				$notificationDialog.dialog("open");
			},
			destroy = function () {
				$notificationDialog.html("");
				$notificationDialog.unbind();
				$notificationDialog.dialog("destroy");
				$notificationDialog.remove();
			};
		
		if ($notificationDialog.length === 0) {
			$("body").append('<div id="notification-dialog"></div>');
			$notificationDialog = $("#notification-dialog");
		}
		
		$notificationDialog.html('<ul class="frame"><li class="sub-title-row"><span class="ui-icon"></span><p class="sub-title"></p></li><li class="content"></li></ul>');
		$icon = $notificationDialog.find(".ui-icon");
		$subTitle = $notificationDialog.find(".sub-title");
		$content = $notificationDialog.find(".content");
		
		$notificationDialog.dialog({
			autoOpen: false,
			modal: true,
			resizable: true,
			width: 600,
			height: 400,
			buttons: {
				OK: onOKButtonClick
			}
		});
		
		$notificationDialog.bind("dialogclose", onDialogClose);
		$notificationDialog.addClass("ui-dialogs");
		
		return {
			types: types,
			open: open,
			destroy: destroy,
			$container: $notificationDialog
		};
	}();
	
	ns.details = function () {
		var 
			$detailsDialog = $("#details-dialog"),
			$accordion = null,
			closeCallback = null,
			onDialogClose = function () {
				var 
					callback = closeCallback;

				closeCallback = null;
				$accordion.accordion("destroy");
				$detailsDialog.html("");
				if (typeof callback === "function") {
					callback.apply(arguments);
				}	
			},
			onOKButtonClick = function () {
				$detailsDialog.dialog("close");
			},
			open = function (html, title, callback) {
				closeCallback = callback;
				$accordion = $(html);
				$detailsDialog.html($accordion);
				$detailsDialog.dialog("option", "title", typeof title === "string" ? title : "Details");
				$accordion.accordion({
					collapsible: true,
					active: false,
					animated: false,
					autoHeight: false
				});
				$detailsDialog.dialog("open");
			},
			destroy = function () {
				$detailsDialog.html("");
				$detailsDialog.unbind();
				$detailsDialog.dialog("destroy");
				$detailsDialog.remove();
			};
		
		if ($detailsDialog.length === 0) {
			$("body").append('<div id="details-dialog"></div>');
			$detailsDialog = $("#details-dialog");
		}
		
		$detailsDialog.dialog({
			autoOpen: false,
			modal: true,
			resizable: true,
			width: 650,
			height: 600,
			buttons: {
				OK: onOKButtonClick
			}
		});
		
		$detailsDialog.bind("dialogclose", onDialogClose);
		$detailsDialog.addClass("ui-dialogs");
		
		return {
			open: open,
			destroy: destroy,
			$container: $detailsDialog
		};
	}();
}(window.dialogs = window.dialogs || {}, jQuery));