var resourcePolicy = {
	// sample data
	'mbs.resman.group.a.maxThreads' : '1',
	'mbs.resman.group.a.maxSockets' : '2',
	'mbs.resman.group.a.maxDisk' : '3',
	'mbs.resman.group.a.maxCPU' : '4',
	'mbs.resman.group.a.oldMemSize' : '5',
	'mbs.resman.group.a.newMemSize' : '6',
	'mbs.resman.group.a.maxViolations' : '7',
	'mbs.resman.group.a.policyAction' : 'uninstall',
	'mbs.resman.level.1.policyAction' : 'blacklist',
	'mbs.resman.mbs.bundle.x.maxCPU' : '100',
	'mbs.resman.mbs.bundle.y.maxThreads' : '13',
	'mbs.resman.grouplevel.12.maxDisk' : '4000000',
	'mbs.resman.maxDisk' : '4000000000',
	'mbs.resman.mbs.bundle.x.group' : 'a',
	'mbs.resman.mbs.bundle.z.group' : 'a'
}
var resourcePolicyGroups = {}
var resourcePolicyGroupsRev = {}

String.prototype.startsWith = function(s) {	return this.indexOf(s) == 0 }
String.prototype.endsWith   = function(s) { return this.length - this.lastIndexOf(s) == s.length }

var policyEditor = false; // dialog

var policyEditing = false; // label

var policyName = false; // select
var policyLevel = false; // option
var policyGroupsNamed = false; // option
var policyGroupsLevel = false; // option
var policyBundles = false; // option

var policyNewType = false; // select
var policyNewArg = false; // input

var maxThreads = false; // input
var maxSockets = false; // input
var maxDisk = false; // input
var maxCPU = false; // input
var oldMemSize = false; // input
var newMemSize = false; // input
var maxViolations = false; // input
var policyAction = false; // select
var groupBundlesRow = false; // table row
var groupBundles = false; // text area

// constants
var ACTION_NAMES = [ 'stop', 'disable', 'uninstall', 'blacklist' ];
var ATTRIBUTE_NAMES = [ '.maxThreads', '.maxSockets', '.maxDisk', '.maxCPU', '.oldMemSize', '.newMemSize', '.maxViolations', '.policyAction' ];

function _fillPolicyAttributes(prefix) {
	policyEditing.text(prefix);

	var x = resourcePolicy[(prefix + '.maxThreads')];
	maxThreads.val(x ? x : '');
	x = resourcePolicy[(prefix + '.maxSockets')];
	maxSockets.val(x ? x : '');
	x = resourcePolicy[(prefix + '.maxDisk')];
	maxDisk.val(x ? x : '');
	x = resourcePolicy[(prefix + '.maxCPU')];
	maxCPU.val(x ? x : '');
	x = resourcePolicy[(prefix + '.oldMemSize')];
	oldMemSize.val(x ? x : '');
	x = resourcePolicy[(prefix + '.newMemSize')];
	newMemSize.val(x ? x : '');
	x = resourcePolicy[(prefix + '.maxViolations')];
	maxViolations.val(x ? x : '');
	x = resourcePolicy[(prefix + '.policyAction')];
	if (x) {
		var idx = $.inArray( x, ACTION_NAMES );
		if (idx > 0) x = idx;
	}
	
	// add bundles
	groupBundlesRow.addClass('ui-helper-hidden');
	if (prefix.startsWith('mbs.resman.group.')) {
		groupBundlesRow.removeClass('ui-helper-hidden');

		// update ui
		var data = '';
		var group = prefix.substring(17);
		var bundles = resourcePolicyGroups[group];
		if (bundles) for (var i in bundles)
			if (bundles[i]) data += bundles[i] + '\n';
		groupBundles.val(data);
	}

	policyAction.attr('selectedIndex', x ? x : 0);
}

function _addPolicyOption(value, prefix, added, target) {
	var len = prefix.length; // length of the prefix
	var end = value.lastIndexOf('.');
	var val = value.substring(len, end);
	var pre = value.substring(0, end);
	if( $.inArray(pre, added) == -1) {
		target.append('<option value="' + pre + '">' + val + '</option>');
		added.push(pre);
	}
}
function _prepareGroups(data) {
	resourcePolicyGroups    = {}
	resourcePolicyGroupsRev = {}
	for (var i in resourcePolicy) {
		var dotIndex = i.lastIndexOf('.');
		if (dotIndex == -1) continue;
		if (dotIndex != 10 && i.endsWith('.group')) { // prefix != mbs.resman.<Prop> (not system but bundle policy)
			var bsn = i.substring(11, dotIndex);
			var group = resourcePolicy[i];
			var bundles = resourcePolicyGroups[group];
			if ( !$.isArray(bundles) ) resourcePolicyGroups[group] = bundles = new Array();
			if ( $.inArray(bsn, bundles) == -1 ) bundles.push(bsn);
			resourcePolicyGroupsRev[bsn] = group;
		}
	}
}
function _processPolicy(data) {
	resourcePolicy = data; // save properties locally

	// update UI
	policyLevel.empty();
	policyGroupsNamed.empty();
	policyGroupsLevel.empty();
	policyBundles.empty();

	// update the options
	_prepareGroups(data);
	var added = new Array();
	for (var i in resourcePolicy) {
		var dotIndex = i.lastIndexOf('.');
		if (dotIndex == -1) continue;

		if (i.startsWith('mbs.resman.groupdef.') || i.startsWith('mbs.resman.default.')) { 
			// default & group default cannot be modified, so nothing to do here
		} else if (i.startsWith('mbs.resman.group.')) {
			_addPolicyOption(i, 'mbs.resman.group.', added, policyGroupsNamed);
		} else if (i.startsWith('mbs.resman.grouplevel.')) {
			_addPolicyOption(i, 'mbs.resman.grouplevel.', added, policyGroupsLevel);
		} else if (i.startsWith('mbs.resman.level.')) {
			_addPolicyOption(i, 'mbs.resman.level.', added, policyLevel);
		} else if (dotIndex != 10) { // prefix != mbs.resman.<Prop> (not system but bundle policy)
			var bsn = i.substring(11, dotIndex);
			if (!resourcePolicyGroupsRev[bsn]) _addPolicyOption(i, 'mbs.resman.', added, policyBundles);
		}
	}
	policyName.change();
}
function _requestPolicy() {
	$.post(pluginRoot + '/getPolicy', false, _processPolicy, 'json');
}
function _saveProperty(field, prefix) {
	var val = field.val();
	var id = field.attr('id');
	if (val) {
		if (isNaN(val)) {
			alert('Expected an integer number for the value of property named: ' + id);
			return false;
		} else {
			resourcePolicy[prefix + '.' + id] = val;
		}
	}
	return true;
}
function _savePolicy() {
	var prefix = policyEditing.text();
	var ok = _saveProperty(maxThreads, prefix) && 
		_saveProperty(maxSockets, prefix) &&
		_saveProperty(maxDisk, prefix) &&
		_saveProperty(maxCPU, prefix) &&
		_saveProperty(oldMemSize, prefix) &&
		_saveProperty(newMemSize, prefix) &&
		_saveProperty(maxViolations, prefix);

	if (ok) { // validation is succesfull
		// append bundles if it is a named group
		if (prefix.startsWith('mbs.resman.group.')) {
			var group = prefix.substring(17);
			var bsn = groupBundles.val().toString().replace(/[,;\s]+/g, ',').split(',');
			for (var i in bsn) if (bsn[i]) resourcePolicy[ 'mbs.resman.' + bsn[i] + '.group'] = group;
		}

		// append action and post data
		resourcePolicy[prefix + '.policyAction'] = policyAction.val();
		$.post(pluginRoot + '/savePolicy', resourcePolicy);
		policyEditor.dialog('close');
	}
}

$(document).ready(function() {
	maxThreads   = $('#maxThreads');
	maxSockets   = $('#maxSockets');
	maxDisk      = $('#maxDisk');
	maxCPU       = $('#maxCPU');
	oldMemSize   = $('#oldMemSize');
	newMemSize   = $('#newMemSize');
	maxViolations= $('#maxViolations');
	policyAction = $('#policyAction');
	policyNewArg = $('#policyNewArg');
	policyEditing= $('#policyEditing');
	policyLevel  = $('#policyLevel');
	policyBundles= $('#policyBundles');
	policyGroupsNamed = $('#policyGroupsNamed');
	policyGroupsLevel = $('#policyGroupsLevel');
	groupBundlesRow   = $('#groupBundlesRow');
	groupBundles = $('#groupBundles');

	var _buttons = {};
	_buttons[i18n.e_save] = _savePolicy;
	policyEditor = $('#policyEditor').dialog({
		autoOpen: false,
		modal: true,
		title: i18n.e_title,
		width: '65em',
		buttons: _buttons,
		open: _requestPolicy
	});

	// init change handlers
	policyNewType = $('#policyNewType').change(function() {
		switch($(this).attr('selectedIndex')) {
			case 0:
			case 2:
				policyNewArg.val('<level>');
				break;
			case 1:
				policyNewArg.val('<group name>');
				break;
			case 3:
				policyNewArg.val('<bundle symbolic name>');
				break;
		}
	}).change();
	policyName = $('#policyName').change(function() {
		_fillPolicyAttributes($(this).val());
	}).change();

	// add button handlers
	$('#openPE').click(function() {
		policyEditor.dialog('open');
		policyName.change();
	});
	$('#policyEdit').click(function() {
		policyName.change();
	});
	$('#policyDelete').click(function() {
		var prefix = policyEditing.text();
		for (var i in ATTRIBUTE_NAMES) {
			delete resourcePolicy[ prefix + ATTRIBUTE_NAMES[i] ];
		}
		// delete bundles from the group
		if( prefix.startsWith('mbs.resman.group.') ) {
			var group = prefix.substring(17);
			var bundles = resourcePolicyGroups[group];
			if (bundles) for (var i in bundles)
				delete resourcePolicy[ 'mbs.resman.' + bundles[i] + '.group'];
		}

		alert(i18n.e_delWarn.msgFormat(prefix));

		// remove entry from the drop-down box, if not from defaults
		var index = policyName.attr('selectedIndex');
		if (index > 2 ) policyName.find('option:selected').remove();
		policyName.attr('selectedIndex', 0).change();
	});
	$('#policyNew').click(function() {
		var arg = policyNewArg.val();
		if ( !arg ) {
			alert(i18n.e_noArg);
			return;
		}
		switch( policyNewType.attr('selectedIndex') ) {
			case 0:
			case 2:
				if ( isNaN(arg) ) {
					alert(i18n.e_noNum);
					return false;
				}
			default:
				if ( arg.match(/[\\.\w]+/) != arg ) {
					alert(e_argSpace);
					return false;	
				}
		}
		var prefix = policyNewType.val() + arg;
		_fillPolicyAttributes(prefix);
	});
});

