// main Ajax handler
var tab_selected = 0;
var json = []
/* ploting functions */
var plotOptions = {
	xaxis: { 
		mode: 'time',
		ticks: 5
	},
	yaxis: { 
		tickFormatter: false,
		tickSize: null,
		min: 0
	},
	selection: { 
		mode: 'x' 
	},
	legend: {
		container: null,
		backgroundOpacity: .3,
		noColumns: 3
	}
}
function shorterName(label) {
	var x = label.lastIndexOf('/');
	if (x!=-1) label = label.substring(x+1);
	x = label.lastIndexOf('\\');
	if (x!=-1) label = label.substring(x+1);
	return label;
}
function updateState() {
	$.post(pluginRoot + '/ping', false, updateButtons);
	setTimeout('updateState()', 5000);
}
function updateButtons(started) {
	if('true' == started) {
		$('#start').addClass('ui-state-disabled');
		$('#stop').removeClass('ui-state-disabled');
	} else {
		$('#start').removeClass('ui-state-disabled');
		$('#stop').addClass('ui-state-disabled');
	}
}
function addToMap(x, y, chart_data_map, ctx) {
	var val = chart_data_map[ctx];
	if (!val) {
		val = {
			label: ctx,
			data: new Array()
		}
		chart_data_map[ctx] = val;
	}
	val.label = ctx + ' (' + y + ')';
	// append data series
	val.data.push( new Array(x, y) );
}
function toChartData(data, policyName, peak) {
	if (!data || !policyName) return false;

	// combine series into map
	var chart_data_map = {}; // context_name -> data
	var numSamples = 0;
	for (var i in data) {
		numSamples++;
		var entry = data[i];
		var ctx = shorterName(entry['context_name'].toString());
		if (ctx == 'System Bundle') ctx = 'System';
		var x = entry['ts'];
		var y = entry[policyName];
		if (y) {
			// add system-wide entries
			if (policyName == 'mem_used' && entry['process_mem_used'])
				addToMap(x,new Number(entry['process_mem_used']), chart_data_map, '<b>mBS Mem Usage</b>');
			if (policyName == 'cpu_usage' && entry['process_cpu_usage'])
				addToMap(x,new Number(entry['process_cpu_usage']),chart_data_map, '<b>mBS CPU Usage</b>');
			if (policyName == 'disk_usage' && entry['process_disk_usage'])
				addToMap(x,new Number(entry['process_disk_usage']),chart_data_map, '<b>mBS Storage Size</b>');
			if (policyName == 'num_threads' && entry['process_num_threads'])
				addToMap(x,new Number(entry['process_num_threads']),chart_data_map, '<b>All Threads</b>');

			// add local (and calculate peak values)
			y = new Number(y);
			addToMap(x,y,chart_data_map, ctx);
			if (y>peak.max) { peak.max = y; peak.maxCTX = ctx; }
			if (y<peak.min) { peak.min = y; peak.minCTX = ctx; }
		}
	}
	$('#numSamples').text(numSamples);

	// convert data map to array series
	var chart_data = new Array();
	for (var i in chart_data_map) chart_data.push( chart_data_map[i] );
	return chart_data
}

function showChart(zoom) {
	var target;
	var peakTarget;
	var policy;
	var peak = {
		min: Number.MAX_VALUE, 
		max: Number.MIN_VALUE,
		maxCTX: false,
		minCTX: false
	}
	plotOptions.yaxis.tickFormatter = intFormatter;
	plotOptions.yaxis.tickSize = null;
	switch(tab_selected) {
		case 0:
			target = 'cpu';  policy = 'cpu_usage';
			plotOptions.yaxis.tickFormatter = percentFormatter;
			break;
		case 1:
			target = 'disk'; policy = 'disk_usage';
			plotOptions.yaxis.tickFormatter = byteFormatter;
			break;
		case 2:
			target = 'mem';  policy = 'mem_used';
			plotOptions.yaxis.tickFormatter = byteFormatter;
			break;
		case 3:
			target = 'thrs'; policy = 'num_threads';
			plotOptions.yaxis.tickSize = 1;
			break;
		case 4:
			target = 'sock'; policy = 'num_sockets';
			plotOptions.yaxis.tickSize = 1;
			break;
	}
	var data = toChartData(json, policy, peak);
	if (data.length > 0) {
		var plot = $('#'+target);
		plotOptions.xaxis.ticks = data[0].data.length < 5 ? data.length : 5;
		$.plot(plot, data, zoom ? $.extend(true, {}, plotOptions, {xaxis: zoom}): plotOptions);
		plot.bind('plotselected', function(event, ranges) {
			showChart({ min: ranges.xaxis.from, max: ranges.xaxis.to });
		});
		$('#peak_'+target).html(
			'<h3>' + i18n.peak_title + '</h3>' +
			'<pre>' +
			i18n.peak_min + peak.min.toFixed(0) + '\t(' + peak.minCTX + ')\n' +
			i18n.peak_max + peak.max.toFixed(0) + '\t(' + peak.maxCTX + ')\n' +
			'</pre>'
		);
	} else {
		$('#'+target).html('<b><font color="red">' + i18n.chart_no_data + '</font></b>');
		$('#peak_'+target).text('');
	}
}
function intFormatter(val, axis) { return new Number(val).toFixed(0) }
function percentFormatter(val, axis) { return new Number(val).toFixed(2) + '%'; }
function byteFormatter(val, axis) {
	if (val > 1000000) return (val / 1000000).toFixed(1) + ' MB';
	else if (val > 1000) return (val / 1000).toFixed(1) + ' kB';
	else return val.toFixed(0) + ' B';
}

$(document).ready(function() {
	var dialog = $('#dialog').dialog({
		autoOpen: false,
		modal: true,
		draggable: false,
		resizable: false,
		dialogClass: 'ui-state-error ui-widget',
		width: '50%'
	});
	//plotOptions.legend.container = $('#legend'); // setup legend
	// setup tabs
	$('#tabs').tabs({
		show: function(event, ui) {
			tab_selected = ui.index;
			showChart();
		}
	});
	// hover states on the static widgets
	$('ul#icons li').hover(
		function() { $(this).addClass('ui-state-hover'); }, 
		function() { $(this).removeClass('ui-state-hover'); }
	);
	// reload button click
	$('#reload').click(function() {
		$('#loading').show();
		$.ajax({
			type: 'post',
			url: pluginRoot + '/data',
			dataType: 'json',
			success: function(data) {
				json = data; 
				showChart();
				$('#loading').hide();
			},
			error: function (XMLHttpRequest, textStatus, errorThrown) {
				$('#loading').hide();
				dialog.dialog('open');
			}
		});
	});
	updateState();
	$('#start').click(function() {
		if ( !$(this).hasClass('ui-state-disabled') ) $.post(pluginRoot + '/start', false, updateButtons);
	});
	$('#stop').click(function() {
		if ( !$(this).hasClass('ui-state-disabled') ) $.post(pluginRoot + '/stop', false, updateButtons);
	});
	$('#reset').click(function() {
		if ( !$(this).hasClass('ui-state-disabled') ) {
			$.post(pluginRoot + '/reset', false, updateButtons);
			json = [];
			showChart();
		}
	});
});
