#! /bin/sh
###################################################################################################
# Update from a directory (on a USB key for example).
# 
# 2014 Logilin for FDE
# 20/10/2014 : Modified by FDE <a.courrier@fde.fr>
# 16/12/2014 : Modified by HAGER < m.courivaud@hager.fr>
#
# Versions
#
# 1.0.0 
# Version which already support detection of both firmware and software
# Start update_firmware.sh script in case of firmware update detection
# Copy zipfile to $UPDATE_INSTALL_DIR in case of software update detection
# But for both, the detection stopped after the first one detected
#
# 1.0.1
# Modification on the method used to read the content of the USB key.
# Before a for loop was used that can't handle filenames with spaces 
# Now a while loop is used that do the job normally
#
# 1.0.2
# Modification of some messages so that logfile could be more precise
#
# 1.0.3
# Add signalisation before and after copying update file to the location where 
# it'll be exploited as it may take a long time and the user will wonder what
# happened
# 
# 1.0.4
# Change some message for the logfile to be more understandable afterwards
# 
# 1.0.5
# Add the Writelog function and the UPDATE_FROM_DIRECTORY_SCRIPT_INITIALS
# 
# 1.0.6
# Add some msg to make logs more clear
#
# 1.2.0
# Check presence of "libraries" before sourcing them because sourcing a missing file will abort 
# the script and therefore no msg will be logged
###################################################################################################

UPDATE_FROM_DIRECTORY_SCRIPT_VERSION="1.2.0"
UPDATE_FROM_DIRECTORY_SCRIPT_INITIALS="[UfD]"

source /usr/local/bin/update_common
source /usr/local/bin/BSPUpdateTools

###################################################################################################
# Variables defined for this script
###################################################################################################

HBOX_UPDATE_LOG_FILE=/data/var/log/update

FW_PREFIX=HBoxFirmware

UPDATE_FIRMWARE_SCRIPT="/usr/local/bin/update_firmware.sh"

###################################################################################################
# Script entry
#
# The update source can be either a USB stick or some files copied to /tmp directory,
###################################################################################################
###################################################################################################

LogUpdate "----------------------------- $(basename "$0") v$UPDATE_FROM_DIRECTORY_SCRIPT_VERSION - started"

# Can't remember why the parameter is checked to null as this script is 
# always lunche by "plug_notify.sh" script which always add a parameter

if [ -z "$1" ] ; then
	WORKING_DIR="/tmp/"
else
	WORKING_DIR=$1
fi

# Check working dir presence

if [[ ! -d $WORKING_DIR ]]; then
	LogUpdate "Abort on serious error ($WORKING_DIR is not a directory)"
	update_error
	
	exit 1
fi

cd "$WORKING_DIR"
LogUpdate "Start USB key analysing in `pwd` directory"

##### First of all, looking for a pure firmware file : HBoxFirmware prefix

LogUpdate "Verifyng firmware presence : Looking for a $FW_PREFIX file at $(pwd)"

for fw in `ls -r .`
do

	# Check for an empty folder
	if [ "$fw" = "*" ] ; then
		LogUpdate "Empty directory. Abort update process"
		exit 1
	fi
	
	# Looking for a specific file pattern / ^ ignore case for prefix! :-)
	if echo "$fw" | grep -i "^${FW_PREFIX}-[0-9]\+\.[0-9]\+\.[0-9]\+"'$' > /dev/null
	then
		LogUpdate "HBoxFirmware file found = $fw. Start UPDATE_FIRMWARE_SCRIPT update script"

		/usr/local/bin/update_firmware.sh

		exit $?
	fi

done	

##### Pure firmware file not found, then look for an update file : zipfile

LogUpdate "Firmware file not found, verifyng now update file presence : Looking for a $UPDATE_EXTENSION file at $(pwd)"

ls | while read fw
do

	# Update file must be a zip file
	if echo "$fw" | grep $UPDATE_EXTENSION >/dev/null 2>&1; then
		
		LogUpdate "Update file found = $fw"

		SetLed POWER GREEN FAST_BLINK
		LogUpdate "Copying it to $UPDATE_INSTALL_DIR so that update can start"

		cp "$fw" $UPDATE_INSTALL_DIR 2>&1 >> $HBOX_UPDATE_LOG_FILE

		SetLed POWER GREEN NORMAL_BLINK

		exit $?
	fi

done

LogUpdate "End USB analyzing"
LogUpdate "----------------------------- $(basename "$0") v$UPDATE_FROM_DIRECTORY_SCRIPT_VERSION - ended"
